/*
 * (C) Copyright 2005 NEC Electronics Corporation
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include "boot.h"
#include <config.h>

//; --- SRAM regin
#define	SRAM_BASE	        0xA0000000
#define	SRAM_OFFSET0		0x0007f000
#define	SRAM_OFFSET1		0x00000a00
#define BURST_BARRIER		0xA0000000

/*
 * SyncBurst Mode flag
 */
#define SYNCBURST_MODE_BASE	0xa007f000
#define SYNCBURST_MODE_OFFSET	0x00000a00
#define SYNCBURST_TRUE		0xffffffff
#define SYNCBURST_FALSE		0x00000000

BEGIN_INIT_FUNC(set_flash_burst)

	mov r10, lr

	mov	r1, #0x80000000
	ldr	r0, [r1]
	cmp	r0, #0
	bne	pe12

	ldr r1, FLASH_BURST
	mov r2, #SRAM_BASE
	add r2, r2, #SRAM_OFFSET0
	add r2, r2, #SRAM_OFFSET1
	mov r5, #0
	ldr r6, BURST_SIZE

cpy_sram:
	ldr r0, [r1, r5]
	str r0, [r2, r5]
	add r5, r5, #4
	cmp r5, r6
	bne cpy_sram

	mov r0,  r2
	mov lr,  pc

	/* Jump to SRAM */
	mov pc,  r0
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	mov	lr, r10
pe12:
	mov	pc, lr

FLASH_BURST:	.word set_start
BURST_SIZE:	.word set_end - set_start

set_start:

@; --------------------------------------
@; --- FLASH Burst Mode Setting Start ---
@; --------------------------------------
@--- SMU_FLA_CLK_DLY    (0xC801_01C0) <= 0x0000_1000 1cycle Data Hold
		MOV     r0, 	#0xC8000000
		ADD     r0, r0,	#0x00010000
		mov	r1,	#0x00001000
		STR     r1,[r0,	#0x01C0]

@; ------------------------------------
@; ==== FLASHRCR ====
@; Async			0x0000_BFCF
@; Sync  BL: 4 LC:7 DH:1 BW:1	0x0000_3FC9
@; Sync  BL: 4 LC:6 DH:1 BW:1	0x0000_37C9
@; Sync  BL: 4 LC:5 DH:1 BW:1	0x0000_2FC9
@; Sync  BL: 4 LC:4 DH:1 BW:1	0x0000_27C9
@; Sync  BL: 8 LC:7 DH:1 BW:1	0x0000_3FCA
@; Sync  BL: 8 LC:6 DH:1 BW:1	0x0000_37CA
@; Sync  BL: 8 LC:5 DH:1 BW:1	0x0000_2FCA
@; Sync  BL: 8 LC:4 DH:1 BW:1	0x0000_27CA
@; Sync  BL:16 LC:7 DH:1 BW:1	0x0000_3FCB
@; Sync  BL:16 LC:6 DH:1 BW:1	0x0000_37CB
@; Sync  BL:16 LC:5 DH:1 BW:1	0x0000_2FCB
@; Sync  BL:16 LC:4 DH:1 BW:1	0x0000_27CB
@; Sync  BL:16 LC:4 DH:0 BW:1	0x0000_25CB
@; Sync  BL:16 LC:6 DH:0 BW:1	0x0000_35CB
@; Sync  BL:16 LC:4 DH:0 BW:0	0x0000_25C3  1cycle Data Hold
@; -------------------------------------

@--- SMU_AB0_FLASHRCR     (0xC801_0380) <= 0x0000_25C3
		mov	r5,	#0x00002500
		add	r5, r5, #0x000000C3
		STR     r5,[r0,	#0x0380]

		MOV     r2, 	#0x2F000000
		ADD     r2, r2,	#0x00FF0000

@--- SMU_AB0_FLASHCOMADD0 (0xC801_0384) <= 0x0000_25C3
		STR     r5,[r0,	#0x0384]

@--- SMU_AB0_FLASHCOMADD1 (0xC801_038C) <= 0x0600_25C3
		add	r1, r5, #0x06000000
		STR     r1,[r0,	#0x038C]

@--- SMU_AB0_FLASHCOMDATA0(0xC801_0388) <= 0x0000_0060
		MOV     r1,		#0x00000060
		STR     r1,[r0,	#0x0388]

@--- SMU_AB0_FLASHCOMDATA1(0xC801_0390) <= 0x0000_0003
		MOV     r1,		#0x00000003
		STR     r1,[r0,	#0x0390]

@; ------------------------------------
@; ==== READCTRL ====
@; Async Single		0x0000_0000
@; Async Page		0x0000_0002
@; Sync  Burst		0x0000_0003
@; ------------------------------------
@--- SMU_AB0_CS0READCTRL  (0xC801_0368) <= 0x0000_0003
@---     AB0_FLASHCOMSET  (0x2FFF_0000) <= 0x0000_0041
		MOV     r3,		#0x00000003		@ // Read Mode Set(For AB0)
		MOV     r1,		#0x00000041

@--- ICache Read Wait
		MOV		r6,	#0x00000000		@ Loop Variable set.
ICACHEWAIT_0:
		ADD		r6, r6,	#1
		CMP		r6, 	#0x00001000		
		NOP
		BNE		ICACHEWAIT_0

		STR     r3,[r0,	#0x0368]
		STR     r1,[r2,	#0x0000]

@--- SMU_AB0_CS1READCTRL  (0xC801_036C) <= 0x0000_0003
@---     AB0_FLASHCOMSET  (0x2FFF_0000) <= 0x0000_0042
		MOV     r1,		#0x00000042

@--- ICache Read Wait
		MOV	r6,		#0x00000000		@ Loop Variable set.
ICACHEWAIT_1:
		ADD	r6, r6,	#1
		CMP	r6, 	#0x00001000		
		NOP
		BNE	ICACHEWAIT_1

		STR     r3,[r0,	#0x036C]
		STR     r1,[r2,	#0x0000]

		SET_BASEREG(r5, (0x25C3 << 1))
		SET_BASEREG(r1, 0x03fe0000) ;  // Tail region of CS1 area
		ADD	r1, r1, r5
	
		MOV	r2, #0x60	; // command to Flash
		MOV	r3, #0x3	; // command to Flash
	
		STRH	r2, [r5]	; // set burst mode to CS0 area
		STRH	r3, [r5]

		STRH	r2, [r1]	; // set burst mode to CS1 area
		STRH	r3, [r1]	

@; --------------------------------------
@; --- FLASH Burst Mode Setting End -----
@; --------------------------------------
// --- Sync Burst flag set
	SET_BASEREG(r0, SYNCBURST_MODE_BASE)
	STORE_WORD(r0, SYNCBURST_MODE_OFFSET , SYNCBURST_TRUE, r1)

	mov     pc, lr
set_end:
	nop

END_INIT_FUNC (set_flash_burst)
