/*
 * (C) Copyright 2005-2006 NEC Electronics Corporation
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

	
#include "boot.h"
#include <config.h>

#ifdef	CONFIG_MP200_PLL
# if	( CONFIG_MP200_PLL == 192 )
#  define	PLL_192MHZ
# else
#  if	( CONFIG_MP200_PLL == 150 )
#   define	PLL_150MHZ
#  else
#   error
#  endif
# endif
#endif

//; --- for MP211, system managemant unit
#define	SMU_BASE	        0xc8010000

#define	SMU_RESETCTRL0          0xc8010000
#define	SMU_RESETREQ0           0xc8010004
#define	SMU_RESETREQ0ENA        0xc8010008
#define	SMU_RESETREQ1           0xc801000c
#define	SMU_RESETREQ1ENA        0xc8010010
#define	SMU_RESETCTRL1		0xc8010014

/* base address and range registers for Chip Select 0 to 5 */
/* for I/O or memory devices */
#define	SMU_AB0_CS0BASEADD	0xc8010320
#define	SMU_AB0_CS1BASEADD	0xc8010324
#define	SMU_AB0_CS2BASEADD	0xc8010328
#define	SMU_AB0_CS3BASEADD	0xc801032c
#define	SMU_AB0_CS4BASEADD	0xc8010330
#define	SMU_AB0_CS5BASEADD	0xc8010334
#define	SMU_AB0_CS0BITCOMP	0xc8010338
#define	SMU_AB0_CS1BITCOMP	0xc801033c
#define	SMU_AB0_CS2BITCOMP	0xc8010340
#define	SMU_AB0_CS3BITCOMP	0xc8010344
#define	SMU_AB0_CS4BITCOMP	0xc8010348
#define	SMU_AB0_CS5BITCOMP	0xc801034c

/* Chip Select for Flash Memories */
#define	SMU_AB0_CS0BASEADD_VAL	0
#define	SMU_AB0_CS0BITCOMP_VAL	0xfe000000
#define	SMU_AB0_CS1BASEADD_VAL	0x02000000
#define	SMU_AB0_CS1BITCOMP_VAL	0xfe000000
#ifdef NANDFLASH
# define	SMU_AB0_CS2BASEADD_VAL	0x04000000
# define	SMU_AB0_CS2BITCOMP_VAL	0xfe000000
#endif
	
#ifdef CONFIG_MP200_MINI_BOARD
/* Chip Select for Ether/Wireless LAN/MA-3D/D9930 */
/* access to address range 0x20000000~0x2007ffff asserts CS3 */
#define	SMU_AB0_CS3BASEADD_VAL	0x20000000
#define	SMU_AB0_CS3BITCOMP_VAL	0xfff80000
#else
/* Chip Select for EBC/FPGA itself */
/* access to address range 0x203e0000~0x203effff asserts CS4 */
#define	SMU_AB0_CS4BASEADD_VAL_DEF	0x2ffe0000
#define	SMU_AB0_CS4BASEADD_VAL	0x203e0000
#define	SMU_AB0_CS4BITCOMP_VAL	0xffff0000

/* Chip Select for Ether/Wireless LAN/MA-3D/D9930 */
/* access to address range 0x20000000~0x2007ffff asserts CS5 */
#define	SMU_AB0_CS5BASEADD_VAL	0x20000000
#define	SMU_AB0_CS5BITCOMP_VAL	0xfff80000

/* Chip Select for PCMCIA */
/* access to address range 0x08100000~0x0ffffff asserts CS3 */
#define	SMU_AB0_CS3BASEADD_VAL	0x08100000
#define	SMU_AB0_CS3BITCOMP_VAL	0xfe000000
#endif

/* timing registers for Chip Select 0 to 5 */
/* for I/O or memory devices */
#define	SMU_AB0_CS0WAITCTL		0xc8010350		// flash region
#define	SMU_AB0_CS1WAITCTL		0xc8010354
#define	SMU_AB0_CS2WAITCTL		0xc8010358
#define	SMU_AB0_CS3WAITCTL		0xc801035c
#define	SMU_AB0_CS4WAITCTL		0xc8010360
#define	SMU_AB0_CS5WAITCTL		0xc8010364
#define SMU_AB0_CS0WAITCTL_W		0xc80103a0		// flash region
#define SMU_AB0_CS1WAITCTL_W		0xc80103a4
#ifdef CONFIG_MP200_MINI_BOARD
#define	SMU_AB0_CS3WAITCTL_W		0xc80103ac
#endif


//;;; --- for DSP control

#define	SMU_DSP_ADDRCONV	0xc801021c

//;;; --- for SDRAM control
	
#define	FIF_DDR_BASE		0x4C001000
#define	FIF_DDRConfigA		0x4C001000
#define	FIF_DDRConfigB		0x4C001004
#define	FIF_DDRPowerMng		0x4C001008
#define	FIF_DDRCmdCtrl		0x4C001100
#define	FIF_DDRModeReg		0x4C001110
#define	FIF_ContCmdAddr	

#define	SMU_PLLSEL		0xc8010080
#define	SMU_PLL1CTRL0		0xc8010084
#define	SMU_PLL1CTRL1		0xc8010088
#define	SMU_PLL1CTRL2		0xc801008c
#define	SMU_PLLLOCKTIME		0xc80100a0

#define	SMU_DDR_M1CLKR_DLY	0xC80101D0
#define	SMU_DDR_M4CLKW_DLY	0xC80101D4
#define	SMU_DDR_MCLK_DLY	0xC80101DC
#define	SMU_DDR_DQSOUT_DLY	0xC80101E0
#define	SMU_DDR_DQSOUT12M_DLY	0xC80101E4
#define	SMU_DDR_DQS_I_DLY	0xC80101E8
#define	SMU_DDR_M1CLKR_ST	0xC80101EC	

#define	SMU_BUFSEL0		0xC8010300
#define	SMU_BUFSEL1		0xC8010304

//;;; --- for UART control

#define	SMU_UART_PLLSEL		0xc80100b4
#define	SMU_DIVU70SCLK		0xc8010160
#define	SMU_DIVU71SCLK		0xc8010164
#define	SMU_DIVU72SCLK		0xc8010168

#define	SMU_AB1_U70WAITCTRL	0xC80103C0
#define	SMU_AB1_U71WAITCTRL	0xC80103C4
#define	SMU_AB1_U72WAITCTRL	0xC80103C8
#define	SMU_AB1_U73WAITCTRL	0xC80103CC

#define	SMU_PLL3CTRL0    	0xC80100A8
#define	SMU_PLL3CTRL1    	0xC80100AC

//;;; --- for Automatic Power Control

#define	SMU_AHBCLKCTRL0		0xC8010180
#define	SMU_AHBCLKCTRL1		0xC8010188
#define	SMU_APBCLKCTRL		0xC8010190
#define	SMU_CLKCTRL		0xc80101b8

#define	PB0_BASE		0x40110000
#define	PB0_TI1PCLKCTRL		0x40110000
#define	PB0_A3DPCLKCTRL		0x40110018
#define	PB0_FIFPCLKCTRL		0x4011001c
#define	PB0_ROTCLKCTRL		0x40110020
#define	PB0_DMAPCLKCTRL		0x40110024
#define	PB0_IMGPCLKCTRL		0x40110028
#define	PB0_CAMPCLKCTRL		0x40110030
#define	PB0_PM0PCLKCTRL		0x40110034
#define	PB0_SDIPCLKCTRL		0x40110044
#define	PB0_SWTPCLKCTRL		0x4011004C

#define	PB1_BASE		0xc0090000
#define	PB1_TI0PCLKCTRL		0xC0090000
#define	PB1_INTPCLKCTRL		0xC0090008
#define	PB1_SMUPCLKCTRL		0xC009000c
#define	PB1_LCDPCLKCTRL		0xC0090010
#define	PB1_MWIPCLKCTRL		0xC0090018
#define	PB1_SI0PCLKCTRL		0xC009001c
#define	PB1_PMUPCLKCTRL		0xC0090024

//;;; --- external port
	
#define	SMU_IOCTRL		0xc8010218		// GIO<->U71,U72

//;;; --- for LCD control

#define	SMU_LCD_PLLSEL   	0xC80100B0
#define	SMU_DIVLCDLCLK   	0xC8010134
#define	SMU_CLKEN_POL    	0xC80101BC

#define	LCD_BASE		0xC0040000
#define	LCD_CONTROL		0xC0040000

//;;; --- for clock control

#define	SMU_GCLKCTRL0		0xC8010198
#define	SMU_GCLKCTRL0ENA	0xC801019C
#define	SMU_GCLKCTRL1		0xC80101A0
#define	SMU_GCLKCTRL1ENA	0xC80101A4
#define	SMU_GCLKCTRL2		0xC80101A8
#define	SMU_GCLKCTRL2ENA	0xC80101AC

#define	SWT_BASE		0xcc000000
#define	SWT_TRACE_CLK		0xcc000000

//;;; --- for DSP-debugger <-> ARM core I/F

#define	SWT_PE0_INT_MASK_SET	0x80000040

//;;; --- Auto Frequency Change

#define SMU_AUTO_FREQ_CHANGE	0xC8010194

//;;; ---------------------------------------------------------------------
//;;; -------  Each values
//;;; ---------------------------------------------------------------------

//;;; --- reset control

#define	SMU_RESETREQ0_USB_RST	0x08000000
#define	SMU_RESETREQ0_SDM_RST	0x00100000
#define	SMU_RESETREQ0_PE12_RST	0x0000003c		// PE1,2 reset on pmu drv

#define	SMU_RESETREQ1_MMM_RST	0x00010000
#define	SMU_RESETREQ1_MWI_RST	0x00008000		// uwire enabled on PMU
#define	SMU_RESETREQ1_SI1_RST	0x00000400
#define	SMU_RESETREQ1_MSP_RST	0x00000200
#define	SMU_RESETREQ1_SDI_RST	0x00000100

#define	SMU_RESETREQ0_VAL       0x37eff6c3
#define	SMU_RESETREQ0_RSM_VAL   0x37efffc3
#define	SMU_RESETREQ1_VAL       0x0002bcff
#define	SMU_RESETREQ0ENA_AB1	0x00008000
#define	SMU_RESETREQ0ENA_U70	0x00800000
#define	SMU_RESETREQ0ENA_U71	0x01000000
#define	SMU_RESETREQ0ENA_U72	0x02000000

//;;; --- reset after resume
//;;; ---	clock disabled devices need a reset

#define	SMU_RESETREQ0_RESUME_L1_VAL	(SMU_RESETREQ0_USB_RST | \
					 SMU_RESETREQ0_SDM_RST)
	
#define	SMU_RESETREQ1_RESUME_L1_VAL	(SMU_RESETREQ1_MMM_RST | \
					 SMU_RESETREQ1_SI1_RST | \
					 SMU_RESETREQ1_MSP_RST | \
					 SMU_RESETREQ1_SDI_RST)

#define	SMU_RESETCTRL1_RESUME_L1_VAL	1

#define	SWT_PE0_INT_MASK_SET_VAL	0x00000007

//;;; ---------------------------------------------------------------------

#define	SMU_DSP_ADDRCONV_VAL	0x00000001

//;;; ---------------------------------------------------------------------
//;;; AB0 wait ctrl
#define	SMU_AB0_CS0WAITCTL_VAL	AB0CS01WAIT
#define	SMU_AB0_CS1WAITCTL_VAL	AB0CS01WAIT
#ifdef CONFIG_MP200_MINI_BOARD
#define	SMU_AB0_CS3WAITCTL_VAL	AB0CS3WAIT
#endif
#define	SMU_AB0_CS0WAITCTL_W_VAL	(AB0CS01WAIT & ~0xff0000ff)
#define	SMU_AB0_CS1WAITCTL_W_VAL	(AB0CS01WAIT & ~0xff0000ff)
#ifdef CONFIG_MP200_MINI_BOARD
#define	SMU_AB0_CS3WAITCTL_W_VAL	(AB0CS3WAIT & ~0xff0000ff)
#endif
#define	SMU_AB0_CS5WAITCTL_VAL	AB0CS01WAIT
#define	EBC_WRZ_WAIT_VAL	((AB0CS01WAIT & 0x0000ff00) >> 8)

//;;; -- for SI1 SCLK (Clr Board Validation Mode)
#define	SMU_GCLKCTRL2_VAL_SI1		0xffffffbf
#define	SMU_GCLKCTRL2ENA_VAL_SI1	0x00000040
#define	SMU_GCLKCTRL2ENA_VAL_CLR	0xffffffff


#ifdef	ONDEMAND_CLK
# define	SMU_GCLKCTRL0ENA_VAL_CLR	0xffffffff
# define	SMU_GCLKCTRL1ENA_VAL_CLR	0xffffffff
//#ifdef	SUSPEND_RESUME
//# define	SUSPEND_RESUME_USBCLKON
//#endif
//;;; -- for GCLKCTRL0
# define	SMU_GCLKCTRL0_LCD_LCLK	0x00040000
# define	SMU_GCLKCTRL0_LCD_PCLK	0x00020000
# define	SMU_GCLKCTRL0_LCD_CLK	0x00010000
# define	SMU_GCLKCTRL0_IMG_PCLK	0x00000800
# define	SMU_GCLKCTRL0_IMG_CLK	0x00000400
# define	SMU_GCLKCTRL0_MSE_CLK	0x00000040
# ifdef SUSPEND_RESUME_USBCLKON
//;;;   to prevent lost reset on MSE
#  define	SMU_GCLKCTRL0ENA_VAL_ONDEMAND	(0)
#  define	SMU_GCLKCTRL0ENA_RSM_VAL_ONDEMAND	(0)
# else
#  define	SMU_GCLKCTRL0ENA_VAL_ONDEMAND	(SMU_GCLKCTRL0_MSE_CLK |\
	SMU_GCLKCTRL0_LCD_LCLK | SMU_GCLKCTRL0_LCD_PCLK |\
	SMU_GCLKCTRL0_LCD_CLK | SMU_GCLKCTRL0_IMG_PCLK | SMU_GCLKCTRL0_IMG_CLK)
#  define	SMU_GCLKCTRL0ENA_RSM_VAL_ONDEMAND	(SMU_GCLKCTRL0_MSE_CLK)
# endif
# define	SMU_GCLKCTRL0_VAL_ONDEMAND	(~SMU_GCLKCTRL0ENA_VAL_ONDEMAND)
# define	SMU_GCLKCTRL0_RSM_VAL_ONDEMAND	(~SMU_GCLKCTRL0ENA_RSM_VAL_ONDEMAND)
//;;; -- for GCLKCTRL1
# define	SMU_GCLKCTRL1_MSP_PCLK	0x80000000
# define	SMU_GCLKCTRL1_MSP_SCLK	0x40000000
# define	SMU_GCLKCTRL1_SDI_PCLK	0x20000000
# define	SMU_GCLKCTRL1_SDI_SCLK	0x10000000
# define	SMU_GCLKCTRL1_USB_SCLK	0x00080000
# define	SMU_GCLKCTRL1_USB_CLK	0x00040000

# ifdef SUSPEND_RESUME_USBCLKON
#  define	SMU_GCLKCTRL1ENA_VAL_ONDEMAND	(SMU_GCLKCTRL1_MSP_PCLK |\
	SMU_GCLKCTRL1_MSP_SCLK | SMU_GCLKCTRL1_SDI_PCLK |\
	SMU_GCLKCTRL1_SDI_SCLK)
# else // SUSPEND_RESUME_USBCLKON
#  define	SMU_GCLKCTRL1ENA_VAL_ONDEMAND	(SMU_GCLKCTRL1_MSP_PCLK |\
	SMU_GCLKCTRL1_MSP_SCLK | SMU_GCLKCTRL1_SDI_PCLK |\
	SMU_GCLKCTRL1_SDI_SCLK | SMU_GCLKCTRL1_USB_SCLK |\
	SMU_GCLKCTRL1_USB_CLK)
# endif
# define	SMU_GCLKCTRL1_VAL_ONDEMAND	(~SMU_GCLKCTRL1ENA_VAL_ONDEMAND)

//;;; -- for GCLKCTRL2
# define	SMU_GCLKCTRL2_LCD_CLK	0x40000000
# define	SMU_GCLKCTRL2_SDM_CLK	0x00800000
# define	SMU_GCLKCTRL2_SDM_SCLK	0x00400000
# define	SMU_GCLKCTRL2_MMM_CLK	0x00000200
# define	SMU_GCLKCTRL2_MWI_SCLK	0x00000100
# define	SMU_GCLKCTRL2_SI1_CLK	0x00000040

# define	SMU_GCLKCTRL2ENA_VAL_ONDEMAND	(SMU_GCLKCTRL2_SDM_CLK |\
			SMU_GCLKCTRL2_SDM_SCLK | SMU_GCLKCTRL2_MMM_CLK |\
			SMU_GCLKCTRL2_SI1_CLK | SMU_GCLKCTRL2_LCD_CLK)
# define	SMU_GCLKCTRL2ENA_RSM_VAL_ONDEMAND	(SMU_GCLKCTRL2_SDM_CLK |\
			SMU_GCLKCTRL2_SDM_SCLK | SMU_GCLKCTRL2_MMM_CLK |\
			SMU_GCLKCTRL2_SI1_CLK )

# define	SMU_GCLKCTRL2_VAL_ONDEMAND	(~SMU_GCLKCTRL2ENA_VAL_ONDEMAND)
# define	SMU_GCLKCTRL2_RSM_VAL_ONDEMAND	(~SMU_GCLKCTRL2ENA_RSM_VAL_ONDEMAND)

# define	SWT_TRACE_CLK_MASK	0x00000001
# define	SWT_TRACE_CLK_VAL	(~SWT_TRACE_CLK_MASK)
#endif

//;;; PLL frequency

#define	SMU_PLLLOCKTIME_VAL	0x00000003	//; lockup 510us
#define	SMU_PLLSEL_VAL   	0x0000000A	//; use PLL,auto standby off,FIF hand shake on
#define	SMU_PLLSEL_STATE_MASK	0x00000003
#ifdef	PLL_192MHZ
# define	SMU_PLL1CTRL0_VAL 	0x0000011f
# define	SMU_PLL1CTRL1_VAL 	0x00000001
#endif
#ifdef	PLL_150MHZ
# define	SMU_PLL1CTRL0_VAL 	0x00000018
# define	SMU_PLL1CTRL1_VAL 	0x00000001
#endif

#define	SMU_PLL1CTRL2_VAL	0x00000000		//; start 192MHz
	
//;;; --- for PLL3 setting
#define	SMU_PLL3CTRL0_VAL	0x0000000D		//; 58.7MHz
#define	SMU_PLL3CTRL1_VAL	0x00000000		//; start

//;;; --- for LCD setting

#define	SMU_LCD_PLLSEL_VAL	0x00000003		//; use 12MHz
#define	SMU_DIVLCDLCLK_VAL	0x00000001		//; LCD_PXCLK 6MHz 
#define	LCD_CONTROL_VAL		0x00000146

#define	SMU_CLKEN_POL_VAL	0x00000040		//; SRC_CLKEN_LCD porality change

//;;; -- for DDR/FIFO setting

#define	SMU_DDR_M1CLKR_ST_VAL	0x0

#if defined(CONFIG_MP200_SDRAM_INTERNAL) && (CONFIG_MP200_PLL == 192)
# define	SMU_BUFSEL1_VAL		0x2000a
#else
# define	SMU_BUFSEL1_VAL		0x3000f
#endif

#if ( CONFIG_MP200_PLL == 150 )
# define	SMU_DDR_DQSOUT_DLY_VAL	0xf
# define	SMU_DDR_M1CLKR_DLY_VAL	0xff
# define	SMU_DDR_M4CLKW_DLY_VAL	0x3f
# define	SMU_DDR_MCLK_DLY_VAL	0x1
# define	SMU_DDR_DQSOUT12M_DLY_VAL	0xff
# define	SMU_DDR_DQS_I_DLY_VAL	0x1ff
#else // CONFIG_MP200_PLL == 192(others not supported)
# define	SMU_DDR_DQSOUT_DLY_VAL	0xff
# define	SMU_DDR_M1CLKR_DLY_VAL	0x1
# define	SMU_DDR_M4CLKW_DLY_VAL	0x3f
# define	SMU_DDR_MCLK_DLY_VAL	0x7
# define	SMU_DDR_DQSOUT12M_DLY_VAL	0x1ff
# ifdef CONFIG_MP200_SDRAM_INTERNAL
#  define	SMU_DDR_DQS_I_DLY_VAL	0x7f
# else
#  define	SMU_DDR_DQS_I_DLY_VAL	0xfff
# endif
#endif
	
//;;; --- for DDR setting
#define	FIF_DDRConfigA_B01_VAL	3
#if (CONFIG_MP200_PLL == 150)
# define	FIF_DDRModeReg_VAL	0x00400033
#elif defined(CONFIG_MP200_SDRAM_INTERNAL) && (CONFIG_MP200_PLL == 192)
# define	FIF_DDRModeReg_VAL	0x00400033
#else // CONFIG_MP200_PLL == 192(others not supported)
# define	FIF_DDRModeReg_VAL	0x00000033
#endif
#define	FIF_DDRPowerMng_VAL	0x80021120
#define	FIF_DDRCmdCtrl_VAL_1	0x0000013B
#define	FIF_DDRCmdCtrl_VAL_2	0x00000335
#define	FIF_DDRCmdCtrl_VAL_RSME1	0x0000013a
#define	FIF_DDRCmdCtrl_VAL_RSME2	0x00000335
#define	FIF_DDRConfigA_VAL	(0x43233110 | FIF_DDRConfigA_B01_VAL)
#ifdef CONFIG_MP200_DDRDRIVE_IS_DEFAULT
#define	FIF_DDRConfigB_VAL	0x00003203	/* DBPartEna = ON is default*/
#else
#define	FIF_DDRConfigB_VAL	0x00803203	/* Set DBPartEna ON */
#endif

//;;; --- for UART settings

#ifdef USE_GIOP2x
# define	SMU_IOCTRL_VAL_B	0x00000000		// U71,U72-->MSE, GIO
#else	// !USE_GIOP2x	
# ifdef USE_MSE_ICE
#  define	SMU_IOCTRL_VAL_B	0x00000026		// MSE, GIO-->U71,U72
# else	// !USE_MSE_ICE
#  define	SMU_IOCTRL_VAL_B	0x00000006		// GIO-->U71,U72
# endif // !USE_MSE_ICE
#endif // !USE_GIOP2x
#define	SMU_IOCTRL_VAL		0x00000100 | SMU_IOCTRL_VAL_B	// use uWire(not use SPI)

#define	SMU_DIVU70SCLK_VAL	0x00000000
#define	SMU_DIVU71SCLK_VAL	0x00000000
#define	SMU_DIVU72SCLK_VAL	0x00000000
#define	SMU_UART_PLLSEL_VAL	0x00000333
#define	SMU_AB1_U70WAITCTRL_VAL	0x00010200
#define	SMU_AB1_U71WAITCTRL_VAL	0x00010200
#define	SMU_AB1_U72WAITCTRL_VAL	0x00010200


//;;; --- for clock control
	
#ifdef	NOSRAM
# define	SMU_GCLKCTRL1_VAL_NOSRAM	0xfffff7ff
# define	SMU_GCLKCTRL1ENA_VAL_NOSRAM	0x00000800
# define	SMU_GCLKCTRL1ENA_VAL_CLR	0
#endif /* !NOSRAM */

//;;; --- for Automatic Power Control

#ifndef CONNECT_ICE
#define	SMU_AHBCLKCTRL0_VAL	0x0001FFFF		/* AutoPowerControl PEx ON */
#define	SMU_AHBCLKCTRL1_VAL	0x000000FF		/* SWT ON */
#else
#define	SMU_AHBCLKCTRL0_VAL	0x0001FFF8		/* AutoPowerControl PEx OFF */
#define	SMU_AHBCLKCTRL1_VAL	0x000000FE		/* SWT OFF */
#endif
#define	SMU_APBCLKCTRL_VAL	0x003FFFFF
#define	SMU_CLKCTRL_VAL		0x0000001F

#define	AB0_BASE		0x2fff0000
#define	AB0_FLASHCOMSET		0x2fff0000
#ifdef CONFIG_MP200_MINI_BOARD
#define	AB0_WAIT_MASK		0x2fff000c
#endif

#define	AB0_FLASHCOMSET_VAL	0x00000013

#ifdef CONFIG_MP200_MINI_BOARD
#define	AB0_WAIT_MASK_VAL	0x00000001	/* 0=MASK ON,1=MASK OFF */
#endif

#ifdef CONFIG_MP200_MINI_BOARD
#define	GIO_BASE		0xc0050000

#define GIO_GP_E0L_VAL		0xffffffff		/* input (L) */
#define GIO_GP_E1L_VAL		0xb2c0b33f		/* output (L) */
#define GIO_GP_E0H_VAL		0xffffffff		/* input (H) */
#define GIO_GP_E1H_VAL		0x70000000		/* output (H) */

#define GIO_GP_E1L		0x0000
#define GIO_GP_E0L		0x0004
#define GIO_GP_OL		0x0008
#define GIO_GP_IIAL		0x0014
#define GIO_GP_IDSL		0x001C
#define GIO_GP_IIRL		0x0028
#define GIO_GP_DT1L		0x002C
#define GIO_GP_DT2L		0x0030

#define GIO_GP_E1H		0x0040
#define GIO_GP_E0H		0x0044
#define GIO_GP_OH		0x0048
#define GIO_GP_IIAH		0x0054
#define GIO_GP_IDSH		0x005C
#define GIO_GP_IIRH		0x0068
#define GIO_GP_DT1H		0x006C
#define GIO_GP_DT2H		0x0070

#define GIO_GP_LAN_RST_SET	0x00040004
#define GIO_GP_LAN_RST_CLR	0x00040000
#else
/*
 * On Board Devices
 */
#define	EBC_BASE		0x203E0000
#define	EBC_LAN_RST		0x203e005C

#define	EBC_DEF_BASE		0x2ffE0000
#define	EBC_WRZ_WAIT		0x2ffe0070

#define	EBC_LAN_RST_VAL		1
#endif

/*
 * SyncBurst Mode flag
 */
#define SYNCBURST_MODE_BASE	0xa007f000
#define SYNCBURST_MODE_OFFSET	0x00000a00
#define SYNCBURST_TRUE		0xffffffff
#define SYNCBURST_FALSE		0x00000000

/*
 * Auto Frequency Change
 */
#define SMU_AUTO_FREQ_CHANGE_VAL 0xE0321001

/*
 * Peripheral init flag
 */
#define	PERIPHERAL_INIT_BASE	0xa007f000
#define	PERIPHERAL_INIT_PE1	0xa007fd00
#define	PERIPHERAL_INIT_PE2	0xa007fd04
#define		PERIPHERAL_NOT_YET	0x00000000
#define		PERIPHERAL_EXECUTE	0x76000000
#define		PERIPHERAL_FINISH	0x00003200

BEGIN_INIT_FUNC (platformsetup)
	.set	REG_PID, 0x80000000
	.set	ICACHE_LINE_SIZE_ORD, 5

	mov	r1, #REG_PID
	ldr	r0, [r1]
	cmp	r0, #0
	bne	.Lreset_pe12

.Lreset_pe0:
	SET_BASEREG(r0, PERIPHERAL_INIT_BASE)
	STORE_WORD(r0, PERIPHERAL_INIT_PE1, PERIPHERAL_EXECUTE, r1)
	STORE_WORD(r0, PERIPHERAL_INIT_PE2, PERIPHERAL_EXECUTE, r1)

	// --- stop NMI to dsp
	SET_BASEREG(r0, REG_PID)
	mov	r1, #(SWT_PE0_INT_MASK_SET_VAL)
	str	r1, [r0, #(SWT_PE0_INT_MASK_SET & 0xfff)]

	// Enable ICache to prevent fetches under delay-control operations
	mrc	p15, 0, r0, c1, c0, 0
	orr	r0, r0, #0x1000
	mcr	p15, 0, r0, c1, c0, 0

	// --- Sync Burst flag init
	SET_BASEREG(r0, SYNCBURST_MODE_BASE)
	STORE_WORD(r0, SYNCBURST_MODE_OFFSET, SYNCBURST_FALSE, r1)

	SET_BASEREG(r0, SMU_BASE)
	
	// --- relase reset
        MVN     r1, #0x00000000
        STR     r1, [r0, #(SMU_RESETREQ0ENA & 0x00ff)]
        STR     r1, [r0, #(SMU_RESETREQ1ENA & 0x00ff)]
	LDR	r2, [r0, #(SMU_RESETREQ0 & 0x00ff)]
	AND	r2, r2, #0x3C		/* Read reset status of PE1, PE2 */
	ADD	r2, r2, #(SMU_RESETREQ0_VAL & 0xff000000)
	ADD	r2, r2, #(SMU_RESETREQ0_VAL & 0x00ff0000)
	ADD	r2, r2, #(SMU_RESETREQ0_VAL & 0x0000ff00)
	ADD	r2, r2, #(SMU_RESETREQ0_VAL & 0x000000ff)
	STR	r2, [r0, #(SMU_RESETREQ0 & 0x00ff)]
	STORE_WORD(r0, SMU_RESETREQ1, SMU_RESETREQ1_VAL, r1)

	// --- async bridge setting
	STORE_WORD(r0, SMU_AB0_CS0BASEADD, SMU_AB0_CS0BASEADD_VAL, r1)
	STORE_WORD(r0, SMU_AB0_CS0BITCOMP, SMU_AB0_CS0BITCOMP_VAL, r1)

	STORE_WORD(r0, SMU_AB0_CS1BASEADD, SMU_AB0_CS1BASEADD_VAL, r1)
	STORE_WORD(r0, SMU_AB0_CS1BITCOMP, SMU_AB0_CS1BITCOMP_VAL, r1)

#ifdef	NANDFLASH
	STORE_WORD(r0, SMU_AB0_CS2BASEADD, SMU_AB0_CS2BASEADD_VAL, r1)
	STORE_WORD(r0, SMU_AB0_CS2BITCOMP, SMU_AB0_CS2BITCOMP_VAL, r1)
#endif

#ifdef	SMU_AB0_CS5BASEADD_VAL
	STORE_WORD(r0, SMU_AB0_CS5BASEADD, SMU_AB0_CS5BASEADD_VAL, r1)
	STORE_WORD(r0, SMU_AB0_CS5BITCOMP, SMU_AB0_CS5BITCOMP_VAL, r1)
#endif

#ifdef	SMU_AB0_CS3BASEADD_VAL
	STORE_WORD(r0, SMU_AB0_CS3BASEADD, SMU_AB0_CS3BASEADD_VAL, r1)
	STORE_WORD(r0, SMU_AB0_CS3BITCOMP, SMU_AB0_CS3BITCOMP_VAL, r1)
#endif

#ifdef SMU_AB0_CS4BASEADD_VAL_DEF
	STORE_WORD(r0, SMU_AB0_CS4BASEADD, SMU_AB0_CS4BASEADD_VAL_DEF, r1)
	STORE_WORD(r0, SMU_AB0_CS4BITCOMP, SMU_AB0_CS4BITCOMP_VAL, r1)
#endif

#ifdef	CONFIG_MP200_MINI_BOARD
	SET_BASEREG(r4, SMU_AB0_CS0WAITCTL_VAL)
	SET_BASEREG(r5, SMU_AB0_CS1WAITCTL_VAL)
	SET_BASEREG(r3, SMU_AB0_CS3WAITCTL_VAL)
	mov	r6, #32

	b	1f
	
	.align	ICACHE_LINE_SIZE_ORD
1:
	subs	r6, r6, #1
	bgt	1b
	str	r4, [r0, #(SMU_AB0_CS0WAITCTL & 0xfff)]
	str	r5, [r0, #(SMU_AB0_CS1WAITCTL & 0xfff)]
	str	r3, [r0, #(SMU_AB0_CS3WAITCTL & 0xfff)]
#else
	SET_BASEREG(r2, EBC_DEF_BASE)

	SET_BASEREG(r3, EBC_WRZ_WAIT_VAL)
	SET_BASEREG(r4, SMU_AB0_CS0WAITCTL_VAL)
	SET_BASEREG(r5, SMU_AB0_CS1WAITCTL_VAL)
	mov	r6, #32

	b	1f
	
	.align	ICACHE_LINE_SIZE_ORD
1:
	str	r3, [r2, #(EBC_WRZ_WAIT & 0xfff)]
2:
	subs	r6, r6, #1
	bgt	2b
	str	r4, [r0, #(SMU_AB0_CS0WAITCTL & 0xfff)]
	str	r5, [r0, #(SMU_AB0_CS1WAITCTL & 0xfff)]
#endif

	STORE_WORD(r0, SMU_AB0_CS0WAITCTL_W, SMU_AB0_CS0WAITCTL_W_VAL, r1)
	STORE_WORD(r0, SMU_AB0_CS1WAITCTL_W, SMU_AB0_CS1WAITCTL_W_VAL, r1)
#ifdef	CONFIG_MP200_MINI_BOARD
	STORE_WORD(r0, SMU_AB0_CS3WAITCTL_W, SMU_AB0_CS3WAITCTL_W_VAL, r1)

	SET_BASEREG(r2, AB0_BASE)
	STORE_WORD(r2, AB0_WAIT_MASK, AB0_WAIT_MASK_VAL, r1)
#endif

	SET_BASEREG(r2, AB0_BASE)
	STORE_WORD(r2, AB0_FLASHCOMSET, AB0_FLASHCOMSET_VAL, r1)

	// -- modify CS4BASEADD to simplify address transration rule on linux kernel
#ifdef	SMU_AB0_CS4BASEADD_VAL
	STORE_WORD(r0, SMU_AB0_CS4BASEADD, SMU_AB0_CS4BASEADD_VAL, r1)
#endif
	
	/* Disable unused clock lines */
#ifdef	ONDEMAND_CLK
	SET_BASEREG(r3, SWT_BASE)
	STORE_WORD(r3, SWT_TRACE_CLK, SWT_TRACE_CLK_VAL, r1)

	STORE_WORD(r0, SMU_GCLKCTRL0ENA, SMU_GCLKCTRL0ENA_VAL_ONDEMAND, r1)
	STORE_WORD(r0, SMU_GCLKCTRL0, SMU_GCLKCTRL0_VAL_ONDEMAND, r1)
	STORE_WORD(r0, SMU_GCLKCTRL0ENA, SMU_GCLKCTRL0ENA_VAL_CLR, r1)

	STORE_WORD(r0, SMU_GCLKCTRL1ENA, SMU_GCLKCTRL1ENA_VAL_ONDEMAND, r1)
	STORE_WORD(r0, SMU_GCLKCTRL1, SMU_GCLKCTRL1_VAL_ONDEMAND, r1)
	STORE_WORD(r0, SMU_GCLKCTRL1ENA, SMU_GCLKCTRL1ENA_VAL_CLR, r1)

	STORE_WORD(r0, SMU_GCLKCTRL2ENA, SMU_GCLKCTRL2ENA_VAL_ONDEMAND, r1)
	STORE_WORD(r0, SMU_GCLKCTRL2, SMU_GCLKCTRL2_VAL_ONDEMAND, r1)
	STORE_WORD(r0, SMU_GCLKCTRL2ENA, SMU_GCLKCTRL2ENA_VAL_CLR, r1)

#else	// ONDEMAND_CLK
	STORE_WORD(r0, SMU_GCLKCTRL2ENA, SMU_GCLKCTRL2ENA_VAL_SI1, r1)
	STORE_WORD(r0, SMU_GCLKCTRL2, SMU_GCLKCTRL2_VAL_SI1, r1)
	STORE_WORD(r0, SMU_GCLKCTRL2ENA, SMU_GCLKCTRL2ENA_VAL_CLR, r1)
#endif // ONDEMAND_CLK
		
#ifdef	NOSRAM
	// --- to stop clock lines to the SRAM region	
	STORE_WORD(r0, SMU_GCLKCTRL1ENA, SMU_GCLKCTRL1ENA_VAL_NOSRAM, r1)
	STORE_WORD(r0, SMU_GCLKCTRL1, SMU_GCLKCTRL1_VAL_NOSRAM, r1)
	STORE_WORD(r0, SMU_GCLKCTRL1ENA, SMU_GCLKCTRL1ENA_VAL_CLR, r1)
#endif

	// --- set SDRAM timigs to meet PLL192MHz
	STORE_WORD(r0, SMU_DDR_M1CLKR_DLY, SMU_DDR_M1CLKR_DLY_VAL, r1)
	STORE_WORD(r0, SMU_DDR_M4CLKW_DLY, SMU_DDR_M4CLKW_DLY_VAL, r1)
	STORE_WORD(r0, SMU_DDR_MCLK_DLY, SMU_DDR_MCLK_DLY_VAL, r1)
	STORE_WORD(r0, SMU_DDR_DQSOUT_DLY, SMU_DDR_DQSOUT_DLY_VAL, r1)
	STORE_WORD(r0, SMU_DDR_DQSOUT12M_DLY, SMU_DDR_DQSOUT12M_DLY_VAL, r1)
	STORE_WORD(r0, SMU_DDR_DQS_I_DLY, SMU_DDR_DQS_I_DLY_VAL, r1)
	STORE_WORD(r0, SMU_DDR_M1CLKR_ST, SMU_DDR_M1CLKR_ST_VAL, r1)

	STORE_WORD(r0, SMU_BUFSEL1, SMU_BUFSEL1_VAL, r1)


	SET_BASEREG	(r0, FIF_DDR_BASE)
#if 1	// no needs for elpida DDR
	STORE_WORD(r0, FIF_DDRConfigB, FIF_DDRConfigB_VAL, r1)
#endif
	STORE_WORD(r0, FIF_DDRConfigA, FIF_DDRConfigA_VAL, r1)
	STORE_WORD(r0, FIF_DDRPowerMng, FIF_DDRPowerMng_VAL, r1)
	STORE_WORD(r0, FIF_DDRModeReg, FIF_DDRModeReg_VAL, r1)
	STORE_WORD(r0, FIF_DDRCmdCtrl, FIF_DDRCmdCtrl_VAL_1, r1)

	// --- wait for DDR
	MOV		r1, #0x00000000
DDRINITWAIT:
	ADD		r1, r1, #1
	CMP		r1, #4096
	NOP
	BNE		DDRINITWAIT

	STORE_WORD(r0, FIF_DDRCmdCtrl, FIF_DDRCmdCtrl_VAL_2, r1)

	SET_BASEREG(r0, SMU_BASE)

	// --- start PLL
	MOV	r7,	#(SMU_PLLSEL_STATE_MASK)

	STORE_WORD(r0, SMU_PLLLOCKTIME, SMU_PLLLOCKTIME_VAL, r1)
#ifndef	PLL_192MHZ
	STORE_WORD(r0, SMU_PLL1CTRL0, SMU_PLL1CTRL0_VAL, r1)
	STORE_WORD(r0, SMU_PLL1CTRL1, SMU_PLL1CTRL1_VAL, r1)
#endif
	STORE_WORD(r0, SMU_PLL1CTRL2, SMU_PLL1CTRL2_VAL, r1)
	SET_BASEREG(r1, SMU_PLLSEL_VAL)
	nop
	nop
	nop
	nop

	b	1f
	
	// --- Wait for PLL

	.align	ICACHE_LINE_SIZE_ORD
1:
	STR	r1, [r0, #(SMU_PLLSEL & 0xfff)]

CLK192_WAIT:
	LDR	r6, [r0, #(SMU_PLLSEL & 0xfff)]
	AND	r6, r6,	r7			// r6 <= r6 & r7
	CMP	r6, #(SMU_PLLSEL_VAL & SMU_PLLSEL_STATE_MASK)
	NOP
	BNE	CLK192_WAIT

#ifndef	PLL_192MHZ
	STORE_WORD(r0, SMU_PLL3CTRL1, SMU_PLL3CTRL1_VAL, r1)
#endif	// !PLL_192MHZ

	STORE_WORD(r0, SMU_LCD_PLLSEL, SMU_LCD_PLLSEL_VAL, r1)
	STORE_WORD(r0, SMU_DIVLCDLCLK, SMU_DIVLCDLCLK_VAL, r1)
	STORE_WORD(r0, SMU_CLKEN_POL, SMU_CLKEN_POL_VAL, r1)

	SET_BASEREG(r0, LCD_BASE)
	STORE_WORD(r0, LCD_CONTROL, LCD_CONTROL_VAL, r1)

	//;; for UART console

	SET_BASEREG(r0, SMU_BASE)

	STORE_WORD(r0, SMU_RESETREQ0ENA, SMU_RESETREQ0ENA_AB1, r1)
	STORE_WORD(r0, SMU_RESETREQ0, SMU_RESETREQ0_VAL, r1)

	STORE_WORD(r0, SMU_DIVU70SCLK, SMU_DIVU70SCLK_VAL, r1)
	STORE_WORD(r0, SMU_DIVU71SCLK, SMU_DIVU71SCLK_VAL, r1)
	STORE_WORD(r0, SMU_DIVU72SCLK, SMU_DIVU72SCLK_VAL, r1)

	STORE_WORD(r0, SMU_UART_PLLSEL, SMU_UART_PLLSEL_VAL, r1)
	STORE_WORD(r0, SMU_IOCTRL, SMU_IOCTRL_VAL, r1)

	STORE_WORD(r0, SMU_AB1_U70WAITCTRL, SMU_AB1_U70WAITCTRL_VAL, r1)
	STORE_WORD(r0, SMU_AB1_U71WAITCTRL, SMU_AB1_U71WAITCTRL_VAL, r1)
	STORE_WORD(r0, SMU_AB1_U72WAITCTRL, SMU_AB1_U72WAITCTRL_VAL, r1)

	STORE_WORD(r0, SMU_RESETREQ0ENA, \
			(SMU_RESETREQ0ENA_U70 | \
			 SMU_RESETREQ0ENA_U71 | \
			 SMU_RESETREQ0ENA_U72), r1)

	STORE_WORD(r0, SMU_RESETREQ0, SMU_RESETREQ0_VAL, r1)


#ifndef NO_AUTOPWRCTRL
	/* Enable Automatic Clock Control */
	SET_BASEREG(r0, SMU_BASE)
	STORE_WORD(r0, SMU_APBCLKCTRL, SMU_APBCLKCTRL_VAL, r1)
	STORE_WORD(r0, SMU_AHBCLKCTRL0, SMU_AHBCLKCTRL0_VAL, r1)
	STORE_WORD(r0, SMU_AHBCLKCTRL1, SMU_AHBCLKCTRL1_VAL, r1)
	STORE_WORD(r0, SMU_CLKCTRL, SMU_CLKCTRL_VAL, r1)
#else	// !NO_AUTOPWRCTRL
	SET_BASEREG(r0, SMU_BASE)
#endif	// !NO_AUTOPWRCTRL

	/* Auto Frequency Change */
	STORE_WORD(r0, SMU_AUTO_FREQ_CHANGE, SMU_AUTO_FREQ_CHANGE_VAL, r1)

	STORE_WORD(r0, SMU_DSP_ADDRCONV, SMU_DSP_ADDRCONV_VAL, r1)
/*
 * Initialize On Board Devices
 */
#ifdef	CONFIG_MP200_MINI_BOARD
	/* release Ethernet controller RESET */
	SET_BASEREG(r0, GIO_BASE)
	STORE_WORD(r0, GIO_GP_IIAL, 0, r1)
	STORE_WORD(r0, GIO_GP_IIAH, 0, r1)
	STORE_WORD(r0, GIO_GP_IDSL, 0xffffffff, r1)
	STORE_WORD(r0, GIO_GP_IDSH, 0xffffffff, r1)
	STORE_WORD(r0, GIO_GP_IIRL, 0xffffffff, r1)
	STORE_WORD(r0, GIO_GP_IIRH, 0xffffffff, r1)
	STORE_WORD(r0, GIO_GP_E0L, GIO_GP_E0L_VAL, r1)
	STORE_WORD(r0, GIO_GP_E0H, GIO_GP_E0H_VAL, r1)
	STORE_WORD(r0, GIO_GP_E1L, GIO_GP_E1L_VAL, r1)
	STORE_WORD(r0, GIO_GP_E1H, GIO_GP_E1H_VAL, r1)
#if 0
	STORE_WORD(r0, GIO_GP_OL, GIO_GP_LAN_RST_SET, r1)
#else
	STORE_WORD(r0, GIO_GP_OL, GIO_GP_LAN_RST_CLR, r1)
#endif
#else
	/* release Ethernet controller RESET */
	SET_BASEREG(r0, EBC_BASE)
	STORE_WORD(r0, EBC_LAN_RST, EBC_LAN_RST_VAL, r1)
#endif

	// Disable ICache to avoid cache aliases
	mov	r1, #0x1000
	mvn	r1, r1
	mrc	p15, 0, r0, c1, c0, 0
	and	r0, r0, r1
	mcr	p15, 0, r0, c1, c0, 0

//// -------------------------------------------------------------------------
5:	
	mov	pc, lr

.Lreset_pe12:
	SET_BASEREG(r1, PERIPHERAL_INIT_BASE)
	add	r1, r1, #(PERIPHERAL_INIT_PE1 & 0xf00)
	add	r1, r1, #(PERIPHERAL_INIT_PE1 & 0xff)
	cmp	r0, #1
	beq	_wait_peripheral
	add	r1, r1, #(PERIPHERAL_INIT_PE2 & 0x00ff)
_wait_peripheral:
	ldr	r0, [r1]
	cmp	r0, #PERIPHERAL_FINISH
	beq	_wait_peripheral_out
	mov	r0, #PERIPHERAL_NOT_YET
	str	r0, [r1]
_wait_peripheral_loop:
	ldr	r0, [r1]
	cmp	r0, #PERIPHERAL_FINISH
	bne	_wait_peripheral_loop
_wait_peripheral_out:	
	b	5b

END_INIT_FUNC (platformsetup)

//// -------------------------------------------------------------------------
//// -------------------------------------------------------------------------
//// -------------------------------------------------------------------------

BEGIN_INIT_FUNC (boot_pll_to_warmboot)
	// Enable ICache to prevent fetches under delay-control operations
	mrc	p15, 0, r0, c1, c0, 0
	orr	r0, r0, #0x1000
	mcr	p15, 0, r0, c1, c0, 0

	// --- stop NMI to dsp
	SET_BASEREG(r0, REG_PID)
	mov	r1, #(SWT_PE0_INT_MASK_SET_VAL)
	str	r1, [r0, #(SWT_PE0_INT_MASK_SET & 0xfff)]

	SET_BASEREG(r0, SMU_BASE)
	
	// --- relase reset
        MVN     r1, #0x00000000
        STR     r1, [r0, #(SMU_RESETREQ0ENA & 0x00ff)]
        STR     r1, [r0, #(SMU_RESETREQ1ENA & 0x00ff)]
	STORE_WORD(r0, SMU_RESETREQ0, SMU_RESETREQ0_RSM_VAL, r1)
	STORE_WORD(r0, SMU_RESETREQ1, SMU_RESETREQ1_VAL, r1)
	
	/* Disable unused clock lines */
#ifdef	ONDEMAND_CLK
	SET_BASEREG(r3, SWT_BASE)
	STORE_WORD(r3, SWT_TRACE_CLK, SWT_TRACE_CLK_VAL, r1)

	STORE_WORD(r0, SMU_GCLKCTRL0ENA, SMU_GCLKCTRL0ENA_RSM_VAL_ONDEMAND, r1)
	STORE_WORD(r0, SMU_GCLKCTRL0, SMU_GCLKCTRL0_RSM_VAL_ONDEMAND, r1)
	STORE_WORD(r0, SMU_GCLKCTRL0ENA, SMU_GCLKCTRL0ENA_VAL_CLR, r1)

	STORE_WORD(r0, SMU_GCLKCTRL1ENA, SMU_GCLKCTRL1ENA_VAL_ONDEMAND, r1)
	STORE_WORD(r0, SMU_GCLKCTRL1, SMU_GCLKCTRL1_VAL_ONDEMAND, r1)
	STORE_WORD(r0, SMU_GCLKCTRL1ENA, SMU_GCLKCTRL1ENA_VAL_CLR, r1)

	STORE_WORD(r0, SMU_GCLKCTRL2ENA, SMU_GCLKCTRL2ENA_RSM_VAL_ONDEMAND, r1)
	STORE_WORD(r0, SMU_GCLKCTRL2, SMU_GCLKCTRL2_RSM_VAL_ONDEMAND, r1)
	STORE_WORD(r0, SMU_GCLKCTRL2ENA, SMU_GCLKCTRL2ENA_VAL_CLR, r1)

#else	// ONDEMAND_CLK
	STORE_WORD(r0, SMU_GCLKCTRL2ENA, SMU_GCLKCTRL2ENA_VAL_SI1, r1)
	STORE_WORD(r0, SMU_GCLKCTRL2, SMU_GCLKCTRL2_VAL_SI1, r1)
	STORE_WORD(r0, SMU_GCLKCTRL2ENA, SMU_GCLKCTRL2ENA_VAL_CLR, r1)
#endif // ONDEMAND_CLK
		
	// --- set SDRAM timigs to meet PLL192MHz
	STORE_WORD(r0, SMU_DDR_M1CLKR_DLY, SMU_DDR_M1CLKR_DLY_VAL, r1)
# ifdef	PLL_192MHZ
	STORE_WORD(r0, SMU_DDR_M4CLKW_DLY, SMU_DDR_M4CLKW_DLY_VAL, r1)
# endif
	STORE_WORD(r0, SMU_DDR_MCLK_DLY, SMU_DDR_MCLK_DLY_VAL, r1)
	STORE_WORD(r0, SMU_DDR_DQSOUT_DLY, SMU_DDR_DQSOUT_DLY_VAL, r1)
	STORE_WORD(r0, SMU_DDR_DQSOUT12M_DLY, SMU_DDR_DQSOUT12M_DLY_VAL, r1)
	STORE_WORD(r0, SMU_DDR_DQS_I_DLY, SMU_DDR_DQS_I_DLY_VAL, r1)
	STORE_WORD(r0, SMU_DDR_M1CLKR_ST, SMU_DDR_M1CLKR_ST_VAL, r1)

	STORE_WORD(r0, SMU_BUFSEL1, SMU_BUFSEL1_VAL, r1)

	SET_BASEREG	(r0, FIF_DDR_BASE)
#if 1	// no needs for elpida DDR
	STORE_WORD(r0, FIF_DDRConfigB, FIF_DDRConfigB_VAL, r1)
#endif
	STORE_WORD(r0, FIF_DDRConfigA, FIF_DDRConfigA_VAL, r1)
	STORE_WORD(r0, FIF_DDRPowerMng, FIF_DDRPowerMng_VAL, r1)
	STORE_WORD(r0, FIF_DDRModeReg, FIF_DDRModeReg_VAL, r1)
	STORE_WORD(r0, FIF_DDRCmdCtrl, FIF_DDRCmdCtrl_VAL_RSME1, r1)

	// --- wait for DDR
	
	MOV		r1, #0x00000000
DDRINITWAIT_R:
	ADD		r1, r1, #1
	CMP		r1, #4096
	NOP
	BNE		DDRINITWAIT_R

#if 0
	STORE_WORD(r0, FIF_DDRCmdCtrl, FIF_DDRCmdCtrl_VAL_RSME2, r1)
#endif
		
	SET_BASEREG(r0, SMU_BASE)

	// --- start PLL

	MOV	r7,	#(SMU_PLLSEL_STATE_MASK)

	STORE_WORD(r0, SMU_PLLLOCKTIME, SMU_PLLLOCKTIME_VAL, r1)
#ifndef	PLL_192MHZ
	STORE_WORD(r0, SMU_PLL1CTRL0, SMU_PLL1CTRL0_VAL, r1)
	STORE_WORD(r0, SMU_PLL1CTRL1, SMU_PLL1CTRL1_VAL, r1)
#endif	// !PLL_192MHZ
	STORE_WORD(r0, SMU_PLL1CTRL2, SMU_PLL1CTRL2_VAL, r1)
	SET_BASEREG(r1, SMU_PLLSEL_VAL)
	nop
	nop
	nop
	nop

	b	1f
	
	// --- Wait for PLL

	.align	ICACHE_LINE_SIZE_ORD
1:
	STR	r1, [r0, #(SMU_PLLSEL & 0xfff)]

CLK192_WAIT_R:
	LDR	r6, [r0, #(SMU_PLLSEL & 0xfff)]
	AND	r6, r6,	r7			// r6 <= r6 & r7
	CMP	r6, #(SMU_PLLSEL_VAL & SMU_PLLSEL_STATE_MASK)
	NOP
	BNE	CLK192_WAIT_R

#ifndef	PLL_192MHZ
	STORE_WORD(r0, SMU_PLL3CTRL1, SMU_PLL3CTRL1_VAL, r1)
#endif	// !PLL_192MHZ

	
	//;; for UART console

	SET_BASEREG(r0, SMU_BASE)

	STORE_WORD(r0, SMU_RESETREQ0ENA, SMU_RESETREQ0ENA_AB1, r1)
	STORE_WORD(r0, SMU_RESETREQ0, SMU_RESETREQ0_RSM_VAL, r1)

	STORE_WORD(r0, SMU_DIVU70SCLK, SMU_DIVU70SCLK_VAL, r1)
	STORE_WORD(r0, SMU_DIVU71SCLK, SMU_DIVU71SCLK_VAL, r1)
	STORE_WORD(r0, SMU_DIVU72SCLK, SMU_DIVU72SCLK_VAL, r1)

	STORE_WORD(r0, SMU_UART_PLLSEL, SMU_UART_PLLSEL_VAL, r1)
	STORE_WORD(r0, SMU_IOCTRL, SMU_IOCTRL_VAL, r1)

	STORE_WORD(r0, SMU_AB1_U70WAITCTRL, SMU_AB1_U70WAITCTRL_VAL, r1)
	STORE_WORD(r0, SMU_AB1_U71WAITCTRL, SMU_AB1_U71WAITCTRL_VAL, r1)
	STORE_WORD(r0, SMU_AB1_U72WAITCTRL, SMU_AB1_U72WAITCTRL_VAL, r1)

	STORE_WORD(r0, SMU_RESETREQ0ENA, \
			(SMU_RESETREQ0ENA_U70 | \
			 SMU_RESETREQ0ENA_U71 | \
			 SMU_RESETREQ0ENA_U72), r1)

	STORE_WORD(r0, SMU_RESETREQ0, SMU_RESETREQ0_RSM_VAL, r1)

#ifndef NO_AUTOPWRCTRL
	/* Enable Automatic Clock Control */
	SET_BASEREG(r0, SMU_BASE)
	STORE_WORD(r0, SMU_APBCLKCTRL, SMU_APBCLKCTRL_VAL, r1)
	STORE_WORD(r0, SMU_AHBCLKCTRL0, SMU_AHBCLKCTRL0_VAL, r1)
	STORE_WORD(r0, SMU_AHBCLKCTRL1, SMU_AHBCLKCTRL1_VAL, r1)
	STORE_WORD(r0, SMU_CLKCTRL, SMU_CLKCTRL_VAL, r1)
#else	// !NO_AUTOPWRCTRL
	SET_BASEREG(r0, SMU_BASE)
#endif	// !NO_AUTOPWRCTRL

	/* Auto Frequency Change */
	STORE_WORD(r0, SMU_AUTO_FREQ_CHANGE, SMU_AUTO_FREQ_CHANGE_VAL, r1)

	// Disable ICache to avoid cache aliases
	mov	r1, #0x1000
	mvn	r1, r1
	mrc	p15, 0, r0, c1, c0, 0
	and	r0, r0, r1
	mcr	p15, 0, r0, c1, c0, 0

//// -------------------------------------------------------------------------
	mov	pc, lr

        .end
