/*
 * (C) Copyright 2005-2006 NEC Electronics Corporation
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

/*
 * devide value of PLL,OSC12M those supply clock
 * recommended value is quarter clock(current 12M) of PWC(FPGA)
 * * if clock is big, there is potential messing clock up 
 */

typedef	unsigned int u_int;
typedef unsigned char u_char;

#define DIV_12M		 4					/* 12/4 == 3 */
#define DIV_PLL		32					/* 96/32 (or 99/32) == 3 */

/*
 * address
 */
#define	MP200_SMU_S0_BASE		0xc8010000
#define MWI_ADDRESS			0xc8020000

/* MWI_CONT */
#define CS1_EN                  (1U<<5)
#define CS0_EN                  (1U<<4)
#define CS1                     (1U<<3) /* active value of CS1 is 0 */
#define CS0                     (1U<<2) /* active value of CS0 is 1 */
#define	UWIRE_CS0	0

/*
 * MWI
 */

#define MWI_CONT			(MWI_ADDRESS + 0x00)
#define MWI_CS0				(MWI_ADDRESS + 0x04)
#define MWI_CS1				(MWI_ADDRESS + 0x08)
#define MWI_START			(MWI_ADDRESS + 0x0C)
#define MWI_INTSTATUS			(MWI_ADDRESS + 0x20)
#define MWI_INTRAW			(MWI_ADDRESS + 0x24)
#define MWI_INTFFCLR			(MWI_ADDRESS + 0x28)
#define MWI_INTENSET			(MWI_ADDRESS + 0x2C)
#define MWI_INTENCLR			(MWI_ADDRESS + 0x30)
#define MWI_TXQA			(MWI_ADDRESS + 0x40)
#define MWI_TXQ				(MWI_ADDRESS + 0x50)
#define MWI_RXQ				(MWI_ADDRESS + 0x60)

/* MWI_CS */
#define CS_FIXUP(C) (UWIRE_CS0 == (C) ? (CS0_EN | CS0) : (CS1_EN))
#define MWI_EDGE			(0x01)	/* RX_EDGE(up) | TX_EDGE(down) */

/* read : BITRX_EN | BIT_RX(8) | BITATX_EN | BIT_ATX(8) | RX_EN | TX_EN */
#define MWI_SEQUENTIAL_MODE		(0x27270003)
/* write : BITATX_EN | BIT_ATX(8) | BITDTX_EN | BIT_DTX(8) | TX_EN */
#define MWI_PACKAGE_MODE		(0x00272701)
/* read-write : BITRX_EN | BIT_RX(8) | BITDTX_EN | BIT_DTX(8) | RX_EN | TX_EN */
#define MWI_SIMULTANEOUS_MODE		(0x27002703)

/* Used for MWI_INTFFCLR */
#define MWI_FFCLR			1	/* Interrupts Clear */

/* Used for MWI_START */
#define MWI_START_FLAG			1	/* Transmit Start */

/* Ued for MWI_INTRAW */
#define MWI_RAW				1	/* Interrupts Data Outbreak */

/* SMU */
#define	SMU_RESETREQ1			(MP200_SMU_S0_BASE + 0x000c)
#define SMU_PLLSEL			(MP200_SMU_S0_BASE + 0x0080)
#define SMU_DIVMWISCLK			(MP200_SMU_S0_BASE + 0x015C)
#define SMU_GCLKCTRL2			(MP200_SMU_S0_BASE + 0x01A8)
#define SMU_GCLKCTRL2ENA		(MP200_SMU_S0_BASE + 0x01AC)
#define SMU_IOCTRL			(MP200_SMU_S0_BASE + 0x0218)

#define SMU_MWI_PCLK_GCK		(1U << 7)
#define SMU_MWI_SCLK_GCK		(1U << 8)

#define SMU_PLLSEL0				(1U << 0)
#define SMU_PLLSEL1				(1U << 1)
#define SMU_PLLSEL2				(1U << 2)

#define	SMU_RESETREQ1_MWI_RST		0x00008000
#define	SMU_IOCTRL_MWI			0x00000100


#define inl(a)			(*(volatile unsigned int  *)(a))
#define outl(v,a)		(*(volatile unsigned int  *)(a) = (v))

/*
 * SMU Control
 */

static inline void uwire_pclock_start(void)
{
	outl(SMU_MWI_PCLK_GCK, SMU_GCLKCTRL2ENA);
	outl(SMU_MWI_PCLK_GCK, SMU_GCLKCTRL2);
}

static inline void uwire_pclock_stop(void)
{
	outl(SMU_MWI_PCLK_GCK, SMU_GCLKCTRL2ENA);
	outl(0, SMU_GCLKCTRL2);
}

static inline void uwire_sclock_start(void)
{
	outl(SMU_MWI_SCLK_GCK, SMU_GCLKCTRL2ENA);
	outl(SMU_MWI_SCLK_GCK, SMU_GCLKCTRL2);
}

static inline void uwire_sclock_stop(void)
{
	outl(SMU_MWI_SCLK_GCK, SMU_GCLKCTRL2ENA);
	outl(0, SMU_GCLKCTRL2);
}

static inline void uwire_clock_start(void)
{
	outl((SMU_MWI_PCLK_GCK | SMU_MWI_SCLK_GCK), SMU_GCLKCTRL2ENA);
	outl((SMU_MWI_PCLK_GCK | SMU_MWI_SCLK_GCK), SMU_GCLKCTRL2);
}

static inline void uwire_clock_stop(void)
{
	outl((SMU_MWI_PCLK_GCK | SMU_MWI_SCLK_GCK), SMU_GCLKCTRL2ENA);
	outl(0, SMU_GCLKCTRL2);
}

static inline int uwire_check_resetcontrol(void)
{
  	return (inl(SMU_RESETREQ1) & SMU_RESETREQ1_MWI_RST);
}

/*
   setting frequency devider function
   argument is positive integer 1-256
*/

static inline void uwire_set_divisor(register u_int div)
{
	if ((0 < div) && (div < 257)) {
		outl((div - 1), SMU_DIVMWISCLK);
	}
}

/*
 * uWire I/O
 */

static inline int
__attribute__ ((unused,section (".init.text")))
uwire_write(register int cs, register u_char addr, register u_char data)
{
	uwire_sclock_start();					/* SCLK start */

	/* clear */
	outl(MWI_FFCLR, MWI_INTFFCLR);			/* MWI_STATUS clear */

	/* int clear check */
	while(inl(MWI_INTRAW) & MWI_RAW) {
	}

	/* CS0,CS1 Inactive : cS1_EN | CS0_EN | CS1(1) | CS0(0) */
	outl((CS0_EN | CS1_EN | CS1), MWI_CONT);

	/* start transmit&receive */
	if (UWIRE_CS0 == cs) {
		outl(MWI_EDGE, MWI_CS0);
	} else {
		outl(MWI_EDGE, MWI_CS1);
	}
	outl((MWI_PACKAGE_MODE | CS_FIXUP(cs)), MWI_CONT);

	outl((addr << 24), MWI_TXQA);			/* MSB shift (31-24 <-- 7-0) */
	outl((data << 24), MWI_TXQ);			/* MSB shift (31-24 <-- 7-0) */
	outl(MWI_START_FLAG, MWI_START);

	/* wait end transmit */
	while ((inl(MWI_INTRAW) & MWI_RAW) == 0) {
	}


	outl(MWI_FFCLR, MWI_INTFFCLR);			/* MWI_STATUS clear */
	uwire_sclock_stop();					/* SCLK stop */

	/* int clear check */
	while (inl(MWI_INTRAW) & MWI_RAW) {
	}

	/* Select CS Inactive : CSx_EN | CSx(0or1) */
	if (UWIRE_CS0 == cs) {
		outl(CS0_EN, MWI_CONT);
	} else {
		outl(CS1_EN | CS1, MWI_CONT);
	}

	return data;
}

static inline int
__attribute__ ((unused,section (".init.text")))
uwire_read(register int cs, register u_char addr)
{
	register int data;

	uwire_sclock_start();					/* SCLK start */

	/* clear */
	outl(MWI_FFCLR, MWI_INTFFCLR);			/* MWI_STATUS clear */

	/* int clear check */
	while(inl(MWI_INTRAW) & MWI_RAW) {
	}

	/* CS0,CS1 Inactive : CS1_EN | CS0_EN | CS1(1) | CS0(0) */
	outl((CS0_EN | CS1_EN | CS1), MWI_CONT);

	/* start transmit&receive */
	if (UWIRE_CS0 == cs) {
		outl(MWI_EDGE, MWI_CS0);
	} else {
		outl(MWI_EDGE, MWI_CS1);
	}
	outl((MWI_SEQUENTIAL_MODE | CS_FIXUP(cs)), MWI_CONT);

	outl((addr << 24), MWI_TXQA);			/* MSB shift (31-24 <-- 7-0) */
	outl(MWI_START_FLAG, MWI_START);

	/* wait end transmit */
	while ((inl(MWI_INTRAW) & MWI_RAW) == 0) {
	}

	data = inl(MWI_RXQ) & 0xff;

	outl(MWI_FFCLR, MWI_INTFFCLR);			/* MWI_STATUS clear */
	uwire_sclock_stop();					/* SCLK stop */

	/* int clear check */
	while (inl(MWI_INTRAW) & MWI_RAW) {
	}

	/* Select CS Inactive : CSx_EN | CSx(0or1) */
	if (UWIRE_CS0 == cs) {
		outl(CS0_EN, MWI_CONT);
	} else {
		outl(CS1_EN | CS1, MWI_CONT);
	}

	return data;
}

static inline int
__attribute__ ((unused,section (".init.text")))
uwire_init(void)
{
	uwire_clock_stop();				/* PCLK/SCLK stop */

/*
	Change SP0 -> MWI
*/
	outl(SMU_IOCTRL_MWI, SMU_IOCTRL);

/*
	determin clock and set a devisor at valid value
*/
	if (SMU_PLLSEL1 == (inl(SMU_PLLSEL) & (SMU_PLLSEL0 | SMU_PLLSEL1))) {
		uwire_set_divisor(DIV_PLL);	/* MWI 96M or 99M */
	} else {
		uwire_set_divisor(DIV_12M);	/* MWI 12M */
	}

	/* PCLK stop is possible, but we get going that just in case */
	uwire_pclock_start();			/* PCLK start */

	return 0;
}

static inline void
__attribute__ ((unused,section (".init.text")))
uwire_exit(void)
{
	uwire_clock_stop();
	return;
}
