/*
 * bios_common.h
 *
 * Copyright 2006 Sony Corporation.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

#ifndef __BIOS_COMMON_H__
#define __BIOS_COMMON_H__

#include <common.h>

#define	ICX1087_BIOS	1

/* -------------------------
	common data type define
   ------------------------- */
typedef	int		INT32;
typedef	u_int	UINT32;
typedef u_short	UINT16;
typedef u_char	UINT8;
typedef INT32	BIOS_RESULT;

/* ------------------------
	common constant define
   ------------------------ */
#define BIOS_CLOCK_OSC			11289600	/* OSC clock freq. [ 11.2896MHz] */
#define BIOS_CLOCK_LPLL			192000000	/* LPLL clock freq.[192MHz] */
#define BIOS_CLOCK_HPLL			248000000	/* HPLL clock freq.[250MHz] */
#define BIOS_CLOCK_RTC			32768		/* RTC clock freq. [ 32.768KHz] */

/* ------------------------
	common constant define
   ------------------------ */
#define BIOS_OK				0		/* success */
#define BIOS_ERR_PARAM		-1		/* invalid parameter */
#define BIOS_ERR_CONFIG		-2		/* invalid configuration */
#define BIOS_ERR_STATE		-3		/* invalid status */
#define BIOS_ERR_TIMEOUT	-4		/* timeout */
#define BIOS_ERR_QUEUE_OVER	-5		/* memory overflow */

#define BIOS_NOT_SUPPORT	-100	/* unsupported driver */

/* macro define */
#ifndef NULL
#define NULL				(void *)0		/* NULL */
#endif

#define BIOS_MCR_INPB( addr )			(*(volatile UINT8 *)(addr))			/* 1byte read  */
#define BIOS_MCR_OUTPB( addr, data )	*(volatile UINT8 *)(addr) = data	/* 1byte write */
#define BIOS_MCR_INPH( addr )			(*(volatile UINT16 *)(addr))		/* 2byte read  */
#define BIOS_MCR_OUTPH( addr, data )	*(volatile UINT16 *)(addr) = data	/* 2byte write */
#define BIOS_MCR_INPW( addr )			(*(volatile UINT32 *)(addr))		/* 4byte read  */
#define BIOS_MCR_OUTPW( addr, data )	*(volatile UINT32 *)(addr) = data	/* 4byte write */

#endif
/* End of file */

