/*
 * bios_gio.h
 *
 * Copyright 2006,2007 Sony Corporation.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

#ifndef _BIOS_GIO_H_
#define _BIOS_GIO_H_

#include "bios_common.h"

/**********************************************************************/
/* MP201 GPIO														  */
/**********************************************************************/

/*---------------------------- user setting --------------------------*/
#define	GPIO_GPIO0_LIMIT		0x00000800		/* = PEN2			  */

#define	GPIO_GPIO2_BLTCTL		0x00000002
#define	GPIO_GPIO2_CS_SEL		0x00001000
#define	GPIO_GPIO2_WP			0x00008000

/*---------------------------- GPIO register -------------------------*/
#define	GPIO_GPIO0_EO			0xC0050000
#define	GPIO_GPIO0_EI			0xC0050004
#define	GPIO_GPIO0_DO			0xC0050008
#define	GPIO_GPIO0_DI			0xC005000C

#define	GPIO_GPIO2_EO			0xC0050200
#define	GPIO_GPIO2_EI			0xC0050204
#define	GPIO_GPIO2_DO			0xC0050208
#define	GPIO_GPIO2_DI			0xC005020C


/**********************************************************************/
/* PowerIC GPIO														  */
/**********************************************************************/

/*---------------------------- user setting --------------------------*/
/* REGA */
#define REGA_VGP1PS_NORMAL      (0x0 << 4)
#define REGA_VGP1PS_PS          (0x1 << 4)
#define REGA_VL0PS_NORMAL       (0x0 << 3)
#define REGA_VL0PS_PS           (0x1 << 3)
#define REGA_DDIOPS_PWM         (0x0 << 2)
#define REGA_DDIOPS_ECO         (0x1 << 2)
#define REGA_PSCNTL0_REGL0ON    (0x0 << 1)
#define REGA_PSCNTL0_DCDCL1ON   (0x1 << 1)
#define REGA_PSCNTDDL1_OFF      (0x0 << 0)
#define REGA_PSCNTDDL1_ON       (0x1 << 0) 

#define PWC_REGA_L1_ON_ECO      ( REGA_VGP1PS_NORMAL \
		                    	| REGA_VL0PS_NORMAL \
		                    	| REGA_DDIOPS_ECO   \
		                    	| REGA_PSCNTL0_DCDCL1ON \
		                    	| REGA_PSCNTDDL1_ON )		/* 0x07   */
#define PWC_REGA_L1_OFF_ECO     ( REGA_VGP1PS_NORMAL \
		                    	| REGA_VL0PS_NORMAL \
		                    	| REGA_DDIOPS_ECO   \
		                    	| REGA_PSCNTL0_REGL0ON \
		                    	| REGA_PSCNTDDL1_ON )		/* 0x05   */
#define PWC_REGA_L0_OFF_ECO_PS  ( REGA_VGP1PS_PS \
		                    	| REGA_VL0PS_PS \
		                    	| REGA_DDIOPS_ECO   \
		                    	| REGA_PSCNTL0_REGL0ON \
		                    	| REGA_PSCNTDDL1_OFF )		/* 0x1C   */
/* REG */
#define	PWC_PSCNT1_PWROFF		0x04
#define	PWC_PSCNT1_PWROFF_V3	0x02
#define	PWC_PSCNT1_PWROFF_IRIS	0x00
#define	PWC_PSCNT2_PWROFF		0x00
/* 2006.12.20 fuj : Deleted to set REGGP5 */
/* #define	PWC_PSCNT2_REGGP4		0x0E */
#define	PWC_PSCNT2_REGGP4		0x06

/* GPIO2 */
#define	PWC_GPIO1_HOLD			0x02
#define	PWC_GPIO2_LCD_PON		0x80
#define	PWC_GPIO2_LCD_PCI		0x40
#define	PWC_GPIO2_MUTE			0x10
#define	PWC_GPIO2_HP			0x08
#define	PWC_GPIO2_INIT			0x20
/* 2006.11.24 fuj : set GPPUPD */
#define	PWC_GPIO_GPPUPD3_HP		0x10
#define	PWC_GPIO_GPPUPD4_MUTE	0x00

/* INTC */
#define	PWC_INTC_MASK_KEY		0x03
#define	PWC_INTC_FACT_KEYPAD	0x01
#define	PWC_INTC_FACT_HOLD		0x02
#define	PWC_INTC_FACT_CRADLE	0x80
#define	PWC_INTC_FACT_ADINT		0x01
/* 2007.05.17 fuj : Applied RTC wake-up */
#ifdef CONFIG_ICX1111
#define	PWC_INTC_FACT_RTCINT	0x02
#endif
#define	PWC_DETMOD1L_KEYHLD		0x05
#define	PWC_DETMOD1H_CRDDET		0x40
#define	PWC_DETMOD3L_ADINT		0x01

/* 2007.05.17 fuj : Applied RTC wake-up */
#ifdef CONFIG_ICX1111
/* RTC */
#define	PWC_RTC_RTC1_WALE		0x80
#define	PWC_RTC_RTC1_ALE		0xC0
#define	PWC_RTC_RTC2_AFG		0x03
#define	PWC_RTC_RTC2_ADJ		0x10

#define	PWC_INTC_WRSTMSK_RTCAL	0x01
#endif

/* Power SW */
#define	PWC_PSW_SDETUSB			0x20
#define	PWC_PSW_SDETADP			0x10
#define	PWC_PSW_SDETBAT			0x08
#define	PWC_PSW_SSWUSB			0x04
#define	PWC_PSW_SSWADP			0x02
#define	PWC_PSW_SSWBAT			0x01

/* A/D */
#define	PWC_AD_CH_VBAT			0			
#define	PWC_AD_CH_KEY0			2			
#define	PWC_AD_CH_KEY1			3			
#define	PWC_ADMODE_SINGLE		0x10
#define	PWC_ADMODE_STOP			0x00

/* LSI */
#define	PWC_LSIVER_ES12			0x02
#define	PWC_LSIVER_ES202		0x03

/*---------------------------- GPIO register -------------------------*/
/* register address */
/* POWER */
#define	PWC_REGA				0x02
#define	PWC_PSCNT1				0x04
#define	PWC_PSCNT2				0x06

/* GPIO */
#define	PWC_GPIO_IOSEL1			0x10
#define	PWC_GPIO_IOSEL2			0x12
#define	PWC_GPIO_IOOUT1			0x14
#define	PWC_GPIO_IOIN1			0x15
#define	PWC_GPIO_IOOUT2			0x16
#define	PWC_GPIO_IOIN2			0x17
/* 2006.11.24 fuj : set GPPUPD */
#define	PWC_GPIO_GPPUPD3		0x1C
#define	PWC_GPIO_GPPUPD4		0x1E

/* INTC */
#define	PWC_INTC_FMASK1			0x20
#define	PWC_INTC_FACT1			0x21
#define	PWC_INTC_FMASK3			0x24
#define	PWC_INTC_FACT3			0x25
#define	PWC_INTC_CLRFACT1		0x2C
#define	PWC_INTC_CLRFACT3		0x30
#define	PWC_INTC_DETMOD1L		0x32
#define	PWC_INTC_DETMOD1H		0x34
#define	PWC_INTC_DETMOD3L		0x3A
#define	PWC_INTC_BMASK1			0x3E
#define	PWC_INTC_BMASK3			0x42
#define	PWC_INTC_PWONMSK		0x4C

/* 2007.05.17 fuj : Applied RTC wake-up */
#ifdef CONFIG_ICX1111
/* RTC */
#define	PWC_RTC_MIN_RD			0x73
#define	PWC_RTC_HOUR_RD			0x75
#define	PWC_RTC_WEEK_RD			0x77
#define	PWC_RTC_DAY_RD			0x79
#define	PWC_RTC_MONTH_RD		0x7B
#define	PWC_RTC_YEAR_RD			0x7D
#define	PWC_RTC_RTC1			0x8A
#define	PWC_RTC_RTC1_RD			0x8B
#define	PWC_RTC_RTC2			0x8C
#define	PWC_RTC_RTC2_RD			0x8D

#define	PWC_INTC_WRSTMSK		0x92
#endif

/* Power SW */
#define	PWC_PSW_STATE			0x65

/* A/D */
#define	PWC_INTC_ADDATA			0x4D
#define PWC_INTC_ADMODE			0x4E

/* LSI */
#define	PWC_LSIVER				0x6F


/**********************************************************************/
/* Prototype														  */
/**********************************************************************/
extern void  gioPwc_init( void );
extern void  gioPwc_exit( void );
extern void  gioPwc_iosel( UINT8, UINT8, UINT8 );
extern void  gioPwc_ioout( UINT8, UINT8, UINT8 );
extern void  gioPwc_write( UINT8, UINT8 );
extern UINT8 gioPwc_read( UINT8 );
extern void  gioPwc_init_ad(void);
extern void  gioPwc_exit_ad(void);
extern int   gioPwc_read_ad(UINT8, UINT8 *);

#endif    /* _BIOS_GIO_ */

