/*
 * bios_lcd.c
 *
 * Copyright 2006,2007 Sony Corporation.
 *
 * Based on linux-kernel/drivers/video/mp200_v4l2/mp200_lcdhw.c
 *
 * Copyright (C) NEC Electronics Corporation 2006
 *
 *
 * This program is free software;you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by Free Softwere
 * Foundation; either version 2 of License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warrnty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; If not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 */


#include <common.h>

#ifdef CONFIG_ICX1087

#include "icx1087.h"
#include "bios_smu.h"
#include "bios_lcd.h"
#include "bios_gio.h"
#include "bios_pwm.h"


/* 2007.03.06 fuj : Applied various models */
#ifdef CONFIG_ICX1111
/* Backlight parameters */
LcdParam_t lcd_param_sony = {
	.clk_sel	= SMU_PLLSEL_PLL,
	.clk_div	= 0x00000130,
	.bl_full	= PWM_FULL,
	.bl_inv		= 0,
};

/* H/W operations */
LcdOps_t lcd_ops_sony = {
	.poweron	= lcdInit,
	.poweroff	= lcdExit,
	.stop		= lcdStop,
};
#endif


/* 2007.05.30 fuj : Applied RTC wake-up */
#ifdef CONFIG_ICX1111
/* --------------------------------------------------------------
	Function	: lcd_reset_device
	Abstract	: Reset LCD device
	Input		: none
	Return		: none
   -------------------------------------------------------------- */
void lcd_reset_device( void )
{
	BIOS_MCR_OUTPW( SMU_RESETREQ0_ENA,
					(SMU_RESETDEVICE_TFD | SMU_RESETDEVICE_LCD) );
	BIOS_MCR_OUTPW( SMU_RESETREQ0, 0 );
	BIOS_MCR_OUTPW( SMU_RESETREQ0_ENA, 0 );
}
#endif

/* --------------------------------------------------------------
	Function	: lcd_unreset_device
	Abstract	: Unreset LCD device
	Input		: none
	Return		: none
   -------------------------------------------------------------- */
/* 2007.03.06 fuj : Applied various models */
#ifdef CONFIG_ICX1111
void lcd_unreset_device( void )
#else
static void lcd_unreset_device( void )
#endif
{
	BIOS_MCR_OUTPW( SMU_RESETREQ0_ENA,
					(SMU_RESETDEVICE_TFD | SMU_RESETDEVICE_LCD) );
	BIOS_MCR_OUTPW( SMU_RESETREQ0,
					(SMU_RESETDEVICE_TFD | SMU_RESETDEVICE_LCD) );
	BIOS_MCR_OUTPW( SMU_RESETREQ0_ENA, 0 );
}

/* --------------------------------------------------------------
	Function	: lcd_clkctrl_on
	Abstract	: Start LCD CLK/PCLK, TFD PCLK of GatedClock
	Input		: none
	Return		: none
   -------------------------------------------------------------- */
/* 2007.03.06 fuj : Applied various models */
#ifdef CONFIG_ICX1111
void lcd_clkctrl_on( void )
#else
static void lcd_clkctrl_on( void )
#endif
{
	UINT32 regval, ahbclkctrl1;

	/* LCD CLK */
	regval = BIOS_MCR_INPW( SMU_AHBCLKCTRL0 );	
	regval |= SMU_CLKCTRL_LCD;
	BIOS_MCR_OUTPW( SMU_AHBCLKCTRL0, regval );
	/* LCD,TFD PCLK */
	ahbclkctrl1 = BIOS_MCR_INPW( SMU_AHBCLKCTRL1 );
	BIOS_MCR_OUTPW( SMU_AHBCLKCTRL1, ahbclkctrl1 & ~(SMU_CLKCTRL_PB0|SMU_CLKCTRL_PB1) );
	regval = BIOS_MCR_INPW( SMU_APBCLKCTRL );
	regval |= (SMU_CLKCTRL_LCDPCLK | SMU_CLKCTRL_TFDPCLK);
	BIOS_MCR_OUTPW( SMU_APBCLKCTRL, regval );
	BIOS_MCR_OUTPW( SMU_AHBCLKCTRL1, ahbclkctrl1 );
}

/* --------------------------------------------------------------
	Function	: lcd_clkctrl_off
	Abstract	: Stop LCD CLK/PCLK, TFD PCLK of GatedClock
	Input		: none
	Return		: none
   -------------------------------------------------------------- */
/* 2007.03.06 fuj : Applied various models */
#ifdef CONFIG_ICX1111
void lcd_clkctrl_off( void )
#else
static void lcd_clkctrl_off( void )
#endif
{
	UINT32 regval, ahbclkctrl1;

	/* LCD CLK */
	regval = BIOS_MCR_INPW( SMU_AHBCLKCTRL0 );	
	regval &= ~SMU_CLKCTRL_LCD;
	BIOS_MCR_OUTPW( SMU_AHBCLKCTRL0, regval );
	/* LCD,TFD PCLK */
	ahbclkctrl1 = BIOS_MCR_INPW( SMU_AHBCLKCTRL1 );
	BIOS_MCR_OUTPW( SMU_AHBCLKCTRL1, ahbclkctrl1 & ~(SMU_CLKCTRL_PB0|SMU_CLKCTRL_PB1) );
	regval = BIOS_MCR_INPW( SMU_APBCLKCTRL );
	regval &= ~(SMU_CLKCTRL_LCDPCLK | SMU_CLKCTRL_TFDPCLK);
	BIOS_MCR_OUTPW( SMU_APBCLKCTRL, regval );
	BIOS_MCR_OUTPW( SMU_AHBCLKCTRL1, ahbclkctrl1 );
}

/* --------------------------------------------------------------
	Function	: lcd_clock_start
	Abstract	: Start LCD CLK/PCLK/LCLK, TFD PCLK/LCLK
	Input		: mask	: start clock bit
	Return		: none
   -------------------------------------------------------------- */
/* 2007.03.06 fuj : Applied various models */
#ifdef CONFIG_ICX1111
void lcd_clock_start( UINT32 mask )
#else
static void lcd_clock_start( UINT32 mask )
#endif
{
	BIOS_MCR_OUTPW( SMU_GCLKCTRL0ENA, mask );
	BIOS_MCR_OUTPW( SMU_GCLKCTRL0, mask );
	BIOS_MCR_OUTPW( SMU_GCLKCTRL0ENA, 0 );
}

/* 2007.05.30 fuj : Applied RTC wake-up */
#ifdef CONFIG_ICX1111
/* --------------------------------------------------------------
	Function	: lcd_clock_stop
	Abstract	: Stop LCD CLK/PCLK/LCLK, TFD PCLK/LCLK
	Input		: mask	: stop clock bit
	Return		: none
   -------------------------------------------------------------- */
void lcd_clock_stop( UINT32 mask )
{
	BIOS_MCR_OUTPW( SMU_GCLKCTRL0ENA, mask );
	BIOS_MCR_OUTPW( SMU_GCLKCTRL0, 0 );
	BIOS_MCR_OUTPW( SMU_GCLKCTRL0ENA, 0 );
}
#endif

/* --------------------------------------------------------------
	Function	: lcdStart
	Abstract	: LCD On
	Input		: none
	Output		: none
   -------------------------------------------------------------- */
void lcdStart( void )
{
	BIOS_MCR_OUTPW(LCD_LCDOUT, LCD_LCDOUT_ON);	/* Display ON	  */
}

/* --------------------------------------------------------------
	Function	: lcdStop
	Abstract	: LCD Off
	Input		: none
	Output		: none
   -------------------------------------------------------------- */
void lcdStop( void )
{
	BIOS_MCR_OUTPW(LCD_LCDOUT, 0);				/* Display OFF	  */
}

/* --------------------------------------------------------------
	Function	: lcdInit
	Abstract	: Initialize LCD
	Input		: none
	Output		: none
	Note		: LCD_LCLK_DIV has been set in poweron.S .
				  TFD_LCLK_DIV use default value.
   -------------------------------------------------------------- */
/* 2007.03.06 fuj : Applied various models */
#ifdef CONFIG_ICX1111
int lcdInit( void )
#else
void lcdInit( void )
#endif
{
	unsigned long bltctl;

	/* BLTCTL pull control */
	bltctl = BIOS_MCR_INPW(SMU_PULLCTL_DSPDEBUG);
	BIOS_MCR_OUTPW(SMU_PULLCTL_DSPDEBUG, bltctl & ~SMU_PULLCTL_BLTCTL);

	/* Power ON */
/* 2007.04.03 fuj : Applied various board revision */
#ifdef CONFIG_ICX1111
	if (ICX11xx_CHK_SI_REV(ICX11xx_SI_REV_DOBER))
		gioPwc_write(PWC_PSCNT2, PWC_PSCNT2_REGGP4);
#else
/* 2006.11.30 fuj : modified timing of power on for LCD */
	gioPwc_write(PWC_PSCNT2, PWC_PSCNT2_REGGP4);
#endif
	udelay(500);							/* wait 500us	  */ 

	/* stop GatedClock */
	lcd_clkctrl_off();
	/* start PCLK/SCLK */
	lcd_clock_start(SMU_CLOCKGATE_TFD_PCLK | SMU_CLOCKGATE_TFD_LCLK);
	lcd_clock_start(SMU_CLOCKGATE_LCD_CLK |
					SMU_CLOCKGATE_LCD_PCLK | SMU_CLOCKGATE_LCD_LCLK);
	/* cancel reset */
	lcd_unreset_device();
	/* start GatedClock */
	lcd_clkctrl_on();

	/* set CONTROL register */
	BIOS_MCR_OUTPW(LCD_CONTROL, LCD_CONTROL_VAL);
	/* set Display region register */
	BIOS_MCR_OUTPW(LCD_FRAMESEL, LCD_FRAMESEL_VAL);
	BIOS_MCR_OUTPW(LCD_AREAADR_A, LCD_FRAMEBUF_A);
	BIOS_MCR_OUTPW(LCD_AREAADR_B, LCD_FRAMEBUF_B);
	BIOS_MCR_OUTPW(LCD_HOFFSET, LCD_HOFFSET_VAL);
	BIOS_MCR_OUTPW(LCD_HTOTAL, LCD_HTOTAL_VAL);
	BIOS_MCR_OUTPW(LCD_HAREA, LCD_HAREA_VAL);
	BIOS_MCR_OUTPW(LCD_HEDGE1, LCD_HEDGE1_VAL);
	BIOS_MCR_OUTPW(LCD_HEDGE2, LCD_HEDGE2_VAL);
	BIOS_MCR_OUTPW(LCD_VTOTAL, LCD_VTOTAL_VAL);
	BIOS_MCR_OUTPW(LCD_VAREA, LCD_VAREA_VAL);
	BIOS_MCR_OUTPW(LCD_VEDGE1, LCD_VEDGE1_VAL);
	BIOS_MCR_OUTPW(LCD_VEDGE2, LCD_VEDGE2_VAL);
	/* clear interrupt status */
	BIOS_MCR_OUTPW(LCD_INTENCLR, LCD_INTSTATUS_VAL);
	BIOS_MCR_OUTPW(LCD_INTFFCLR, LCD_INTSTATUS_VAL);

	/* start SYNC */
	BIOS_MCR_OUTPW(LCD_LCDOUT, LCD_LCDOUT_SYNC);
	udelay(60000);							/* wait 60ms	  */ 
	gioPwc_iosel(PWC_GPIO_IOSEL2, PWC_GPIO2_LCD_PON|PWC_GPIO2_LCD_PCI, 1);
	gioPwc_ioout(PWC_GPIO_IOOUT2, PWC_GPIO2_LCD_PON, 1);
	udelay(60000);							/* wait 60ms	  */ 
	gioPwc_ioout(PWC_GPIO_IOOUT2, PWC_GPIO2_LCD_PCI, 1);

	/* clear frame buffer */
	memset((UINT8 *)LCD_FRAMEBUF_A, 0, LCD_FRAME_SIZE);
	memset((UINT8 *)LCD_FRAMEBUF_B, 0, LCD_FRAME_SIZE);

	lcdStart();

/* 2007.03.06 fuj : Applied various models */
#ifdef CONFIG_ICX1111
	return BIOS_OK;
#endif
}

/* 2007.03.06 fuj : Applied various models */
#ifdef CONFIG_ICX1111
/* --------------------------------------------------------------
	Function	: lcdExit
	Abstract	: Power off LCD
	Input		: none
	Output		: none
	Note		: 
   -------------------------------------------------------------- */
void lcdExit( void )
{
	/* device reset */
	gioPwc_iosel(PWC_GPIO_IOSEL2, PWC_GPIO2_LCD_PON|PWC_GPIO2_LCD_PCI, 1);
	gioPwc_ioout(PWC_GPIO_IOOUT2, PWC_GPIO2_LCD_PCI, 0);	/* LCD_PCI = Low	*/
	udelay(33000);							/* wait 33ms	  */ 
	gioPwc_ioout(PWC_GPIO_IOOUT2, PWC_GPIO2_LCD_PON, 0);	/* LCD_PON = Low    */
	udelay(33000);							/* wait 33ms	  */ 

	/* stop clock and reset */
	lcd_clock_stop(SMU_CLOCKGATE_TFD_PCLK | SMU_CLOCKGATE_TFD_LCLK);
	lcd_clock_stop(SMU_CLOCKGATE_LCD_CLK |
                   SMU_CLOCKGATE_LCD_PCLK | SMU_CLOCKGATE_LCD_LCLK);
    lcd_reset_device();
	lcd_clkctrl_off();
}
#endif

/* --------------------------------------------------------------
	Function	: lcdGetFrameNo
	Abstract	: Get frame Number display currently
	Input		: none
	Output		: Frame A/B
	Note		: Not used Frame C
   -------------------------------------------------------------- */
UINT32 lcdGetFrameNo( void )
{
	return (BIOS_MCR_INPW(LCD_FRAMESEL) >> 2) & 0x3;
}

/* --------------------------------------------------------------
	Function	: lcdChangeFrame
	Abstract	: Change frame A/B
	Input		: none
	Output		: none
	Note		: Not used Frame C
   -------------------------------------------------------------- */
void lcdChangeFrame( void )
{
	if((BIOS_MCR_INPW(LCD_FRAMESEL) >> 2) == LCD_FRAMESEL_A)
		BIOS_MCR_OUTPW(LCD_FRAMESEL, LCD_FRAMESEL_B);
	else
		BIOS_MCR_OUTPW(LCD_FRAMESEL, LCD_FRAMESEL_A);
}

/* --------------------------------------------------------------
	Function	: lcdCtrlBL
	Abstract	: Control brightness of Backlight
	Input		: none
	Output		: Frame address
	Note		: Not used Frame C
   -------------------------------------------------------------- */
void lcdCtrlBL( UINT32 bl_val )
{
	BIOS_MCR_OUTPH(GPIO_GPIO2_EO, GPIO_GPIO2_BLTCTL);
	if(bl_val > 0)
		BIOS_MCR_OUTPW(GPIO_GPIO2_DO, 
					   (GPIO_GPIO2_BLTCTL<<16)|(GPIO_GPIO2_BLTCTL&0xFFFF));
	else 
		BIOS_MCR_OUTPW(GPIO_GPIO2_DO, 
					   (GPIO_GPIO2_BLTCTL<<16)|((~GPIO_GPIO2_BLTCTL)&0xFFFF));
}

#endif	/* CONFIG_ICX1087 */ 
