/*
 * bios_lcd.h
 *
 * Copyright 2006,2007 Sony Corporation.
 *
 * Based on linux-kernel/drivers/video/mp200_v4l2/mp200_lcd.h
 *
 * Copyright (C) NEC Electronics Corporation 2006
 *
 *
 * This program is free software;you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by Free Softwere
 * Foundation; either version 2 of License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warrnty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; If not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 */



#ifndef _BIOS_LCD_H_
#define _BIOS_LCD_H_

#include "bios_common.h"

/* option */
#define	CONFIG_LCD_RGB565	0
#define	CONFIG_LCD_RGB666	1

#define CFG_LCD_FORMAT		CONFIG_LCD_RGB666
#define CFG_DATA_FORMAT		CONFIG_LCD_RGB565


/* 2007.03.06 fuj : Applied various models */
#ifdef CONFIG_ICX1111
/*-------------------------- type definition -------------------------*/
typedef struct LcdParam {
	UINT32	clk_sel;
	UINT32	clk_div;
	int		bl_full;
	int		bl_inv;
} LcdParam_t;

extern LcdParam_t lcd_param_sony;
extern LcdParam_t lcd_param_epson;
extern LcdParam_t lcd_param_tmd;
extern LcdParam_t lcd_param_tmd_18qvga;

typedef struct LcdOps {
	int     (*poweron)(void);
	void    (*poweroff)(void);
	void    (*stop)(void);
} LcdOps_t;

extern LcdOps_t lcd_ops_sony;
extern LcdOps_t lcd_ops_epson;
extern LcdOps_t lcd_ops_tmd;
extern LcdOps_t lcd_ops_tmd_18qvga;
#endif


/*---------------------------- user setting --------------------------*/
#define LCD_WIDTH			240				/* Width				  */
#define LCD_HEIGHT			320				/* Height				  */

#if (CFG_LCD_FORMAT == CONFIG_LCD_RGB666)
#define LCD_FRAME_SIZE		((LCD_WIDTH * LCD_HEIGHT * 18) / 8)
#else
#define LCD_FRAME_SIZE		((LCD_WIDTH * LCD_HEIGHT * 16) / 8)
#endif

#if (CFG_DATA_FORMAT == CONFIG_LCD_RGB666)
#define LCD_DATA_SIZE		((LCD_WIDTH * LCD_HEIGHT * 18) / 8)
#else
#define LCD_DATA_SIZE		((LCD_WIDTH * LCD_HEIGHT * 16) / 8)
#endif

#define	LCD_FRAMESEL_A		0x00000001	/* Frame A					  */
#define	LCD_FRAMESEL_B		0x00000002	/* Frame B					  */

/* 2007.03.22 fuj : Applied to change memory size */
#ifdef CONFIG_ICX1111
#define	LCD_FRAMEBUF_A		0x33780000	/* Frame A buffer address	  */
#define	LCD_FRAMEBUF_B		0x337C0000	/* Frame B buffer address	  */

#define	LCD_DATABUF			0x33700000	/* Data buffer for conversion */
#else
#define	LCD_FRAMEBUF_A		0x31780000	/* Frame A buffer address	  */
#define	LCD_FRAMEBUF_B		0x317C0000	/* Frame B buffer address	  */

#define	LCD_DATABUF			0x31700000	/* Data buffer for conversion */
#endif /* CONFIG_ICX1111 */

/* prototype */
/* 2007.03.06 fuj : Applied various models */
#ifdef CONFIG_ICX1111
extern int  lcdInit( void );
extern void lcdExit( void );
#else
extern void lcdInit( void );
#endif
extern void lcdStart( void );
extern void lcdStop( void );
extern UINT32 lcdGetFrameNo( void );
extern void lcdChangeFrame( void );
extern void lcdCtrlBL( UINT32 );

/* 2007.03.06 fuj : Applied various models */
#ifdef CONFIG_ICX1111
extern void lcd_unreset_device( void );
extern void lcd_clkctrl_on( void );
extern void lcd_clkctrl_off( void );
extern void lcd_clock_start( UINT32 mask );
/* 2007.05.30 fuj : Applied RTC wake-up */
extern void lcd_reset_device( void );
extern void lcd_clock_stop( UINT32 mask );
#endif


/*---------------------------- LCD register --------------------------*/
/* register address */
#define	LCD_CONTROL			0xC0040000
#define LCD_LCDOUT			0xC0040008
#define LCD_STATUS			0xC004000C
#define LCD_AREAADR_A		0xC0040010
#define LCD_AREAADR_B		0xC0040014
#define LCD_AREAADR_C		0xC0040018
#define LCD_HOFFSET			0xC004001C
#define LCD_HTOTAL			0xC0040020
#define LCD_HAREA			0xC0040024
#define LCD_HEDGE1			0xC0040028
#define LCD_HEDGE2			0xC004002C
#define LCD_VTOTAL			0xC0040030
#define LCD_VAREA			0xC0040034
#define LCD_VEDGE1			0xC0040038
#define LCD_VEDGE2			0xC004003C
#define LCD_BUSSEL			0xC0040040
#define LCD_FRAMESEL		0xC0040044
#define LCD_GAMMACTRL		0xC0040048
#define LCD_GAMMAADR		0xC004004C
#define LCD_GAMMADATA		0xC0040050
#define LCD_REVERSE			0xC0040054
#define LCD_DATAREQ			0xC0040058
#define LCD_INTSTATUS		0xC0040060
#define LCD_INTRAWSTATUS	0xC0040064
#define LCD_INTENSET		0xC0040068
#define LCD_INTENCLR		0xC004006C
#define LCD_INTFFCLR		0xC0040070
#define LCD_ERRORADR		0xC0040074

/* default setting value */
#if (CFG_LCD_FORMAT == CONFIG_LCD_RGB666)
#define	LCD_CONTROL_VAL		0x00000100		/* FCEN:off, RGB666		  */
#else
#define	LCD_CONTROL_VAL		0x00000140		/* FCEN:off, RGB565		  */
#endif
/* 2007.03.22 fuj : Applied various models */
#ifdef CONFIG_ICX1111
#define	LCD_CONTROL_CLKPOL_FALL	0x00000008	/* CLKPOL:falling edge	  */
#endif
#define	LCD_LCDOUT_OFF		0x00000000		/* LCD Off				  */
#define	LCD_LCDOUT_ON		0x00000001		/* LCD On				  */
#define	LCD_LCDOUT_SYNC		0x00000003		/* Sync					  */

#define LCD_HAREA_VAL		LCD_WIDTH		/* Width				  */
#define LCD_VAREA_VAL		(LCD_HEIGHT+1)	/* Height + 1			  */
#define	LCD_HBLANK_VAL		12				/* Blank of Horizontal	  */
#define	LCD_VBLANK_VAL		12				/* Blank of Vertical	  */
											/* HAREA + blank		  */
#define	LCD_HTOTAL_VAL		(LCD_HAREA_VAL + LCD_HBLANK_VAL + 12)
											/* VAREA + blank		  */
#define	LCD_VTOTAL_VAL		(LCD_VAREA_VAL + LCD_VBLANK_VAL)

#define	LCD_HEDGE1_VAL		8
#define	LCD_HEDGE2_VAL		LCD_HBLANK_VAL
#define	LCD_VEDGE1_VAL		8
#define	LCD_VEDGE2_VAL		10

#if (CFG_LCD_FORMAT == CONFIG_LCD_RGB666) 	/* 18BPP				  */
#define	LCD_HOFFSET_VAL		(LCD_HAREA_VAL * 36 / 16)
#else 										/* 16BPP				  */
#define	LCD_HOFFSET_VAL		(LCD_HAREA_VAL * 32 / 16)
#endif

#define	LCD_FRAMESEL_VAL	LCD_FRAMESEL_A	/* default:Frame A		  */

#define LCD_INTSTATUS_VAL	0x0000001F		/* all					  */

#endif    /* _BIOS_LCD_ */

