/*
 * bios_pwm.c
 *
 * Copyright 2006,2007 Sony Corporation.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

#include <common.h>

#ifdef CONFIG_ICX1087

#include "icx1087.h"
#include "bios_smu.h"
#include "bios_pwm.h"

/* --------------------------------------------------------------
	Function	: pwmStart
	Abstract	: PWM start
	Input		: none
	Output		: none
   -------------------------------------------------------------- */
void pwmStart( void )
{
	if( !(BIOS_MCR_INPW( PWM_CH1_CTRL ) & PWM_START) )
		BIOS_MCR_OUTPW( PWM_CH1_CTRL, PWM_START );
}

/* --------------------------------------------------------------
	Function	: pwmStop
	Abstract	: PWM stop
	Input		: none
	Output		: none
   -------------------------------------------------------------- */
void pwmStop( void )
{
	BIOS_MCR_OUTPW( PWM_CH1_CTRL, PWM_STOP );
}

/* --------------------------------------------------------------
	Function	: pwmInit
	Abstract	: Initialize PWM
	Input		: none
	Output		: none
	Note		: 
   -------------------------------------------------------------- */
void pwmInit( void )
{
	UINT32 regval;

	/* stop clock */
	BIOS_MCR_OUTPW( SMU_GCLKCTRL2ENA, SMU_CLOCKGATE_PWM_SCLK|SMU_CLOCKGATE_PWM_CLK );
	BIOS_MCR_OUTPW( SMU_GCLKCTRL2, 0 );
	BIOS_MCR_OUTPW( SMU_GCLKCTRL2ENA, 0 );

	/* change PCTL to PWM1 */
	regval = BIOS_MCR_INPW( SMU_PCTL_C2 );
	BIOS_MCR_OUTPW( SMU_PCTL_C2, regval & ~SMU_PCTL_C2_PWM );
	regval = BIOS_MCR_INPW( SMU_PULLCTL_P2 );
	BIOS_MCR_OUTPW( SMU_PULLCTL_P2, regval | SMU_PULLCTL_P2_PWM0 );

	/* select clock source */
	BIOS_MCR_OUTPW( SMU_PWM_SCLK_SEL, SMU_PWM_SCLK_SEL_OSC );

	/* start clock */
	BIOS_MCR_OUTPW( SMU_GCLKCTRL2ENA, SMU_CLOCKGATE_PWM_SCLK|SMU_CLOCKGATE_PWM_CLK );
	BIOS_MCR_OUTPW( SMU_GCLKCTRL2, SMU_CLOCKGATE_PWM_SCLK|SMU_CLOCKGATE_PWM_CLK );
	BIOS_MCR_OUTPW( SMU_GCLKCTRL2ENA, 0 );

	/* unreset PWM */
	BIOS_MCR_OUTPW( SMU_RESETREQ0_ENA, SMU_RESETDEVICE_PWM );
	BIOS_MCR_OUTPW( SMU_RESETREQ0, SMU_RESETDEVICE_PWM );
	BIOS_MCR_OUTPW( SMU_RESETREQ0_ENA, 0 );

	/* setting PWM */
	BIOS_MCR_OUTPW( PWM_CH1_DELAY0, PWM_DELAY );
	BIOS_MCR_OUTPW( PWM_CH1_TOTAL0, PWM_TOTAL );
	BIOS_MCR_OUTPW( PWM_CH1_LEDGE0, PWM_LEDGE );
/* 2007.03.06 fuj : Applied various models */
#ifdef CONFIG_ICX1111
	if( icx11xx_lcd_param->bl_inv )
		BIOS_MCR_OUTPW( PWM_CH1_TEDGE0, PWM_LEDGE + 1 );
	else
		BIOS_MCR_OUTPW( PWM_CH1_TEDGE0, PWM_TEDGE );
#else
	BIOS_MCR_OUTPW( PWM_CH1_TEDGE0, PWM_TEDGE );
#endif
	BIOS_MCR_OUTPW( PWM_CH1_LOOP0, 0 );
	BIOS_MCR_OUTPW( PWM_INTENSET, 0 );
	BIOS_MCR_OUTPW( PWM_CH1_MODE, PWM_MODE );
}

/* 2007.05.30 fuj : Applied RTC wake-up */
#ifdef CONFIG_ICX1111
/* --------------------------------------------------------------
	Function	: pwmExit
	Abstract	: Exit PWM
	Input		: none
	Output		: none
	Note		: 
   -------------------------------------------------------------- */
void pwmExit( void )
{
	pwmStop();

	/* stop clock */
	BIOS_MCR_OUTPW( SMU_GCLKCTRL2ENA, SMU_CLOCKGATE_PWM_SCLK|SMU_CLOCKGATE_PWM_CLK );
	BIOS_MCR_OUTPW( SMU_GCLKCTRL2, 0 );
	BIOS_MCR_OUTPW( SMU_GCLKCTRL2ENA, 0 );

	/* reset PWM */
	BIOS_MCR_OUTPW( SMU_RESETREQ0_ENA, SMU_RESETDEVICE_PWM );
	BIOS_MCR_OUTPW( SMU_RESETREQ0, 0 );
	BIOS_MCR_OUTPW( SMU_RESETREQ0_ENA, 0 );
}
#endif

/* --------------------------------------------------------------
	Function	: pwmSetValue
	Abstract	: Initialize PWM
	Input		: val	: 0 - 100
	Output		: none
	Note		: 
   -------------------------------------------------------------- */
void pwmSetValue( int val )
{
	UINT32 tedge;

	if( val > 100 ) return;

/* 2007.03.06 fuj : Applied various models */
#ifdef CONFIG_ICX1111
	if( icx11xx_lcd_param->bl_inv ) {
		if (val == 0)
			tedge = 1;
		else
			tedge = icx11xx_lcd_param->bl_full * val / 100;
	}
	else {
		if( (PWM_TOTAL == icx11xx_lcd_param->bl_full) && (val == 100) )
			tedge = 1;
		else
			tedge = PWM_TOTAL - icx11xx_lcd_param->bl_full * val / 100;
	}
#else
	if( (PWM_TOTAL == PWM_FULL) && (val == 100) )
		tedge = 1;
	else
		tedge = PWM_TOTAL - PWM_FULL * val / 100;
#endif

	pwmStop();
	BIOS_MCR_OUTPW( PWM_CH1_TEDGE0, tedge );
	pwmStart();
}

#endif	/* CONFIG_ICX1087 */ 
