/*
 * bios_pwoff.S
 *
 * Copyright 2006,2007 Sony Corporation.
 *
 * Based on linux-kernel/arch/arm/mach-mp200/sleep.S
 * Based on linux-kernel/arch/arm/mm/proc-arm926.S
 */

/*		
 *
 * Copyright (C) NEC Electronics Corporation 2005-2006
 *
 *
 * This program is free software;you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by Free Softwere
 * Foundation; either version 2 of License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warrnty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; If not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 */

/*
 *  linux/arch/arm/mm/proc-arm926.S: MMU functions for ARM926EJ-S
 *
 *  Copyright (C) 1999-2001 ARM Limited
 *  Copyright (C) 2000 Deep Blue Solutions Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 * These are the low level assembler for performing cache and TLB
 * functions on the arm926.
 *
 *  CONFIG_CPU_ARM926_CPU_IDLE -> nohlt
 */


#include "../boot.h"

#ifdef CONFIG_ICX1087


/*
 *  Macros
 */

#define	VM_EXEC					4

/* FIF registers */
#define	FIF_ADDRESS				0x40070000
#define	FIF_WBUFCTRL			(FIF_ADDRESS + 0x0000)
#define FIF_WBUFSTATE			(FIF_ADDRESS + 0x0004)
#define	FIF_S1_ADDRESS			0x4C000000
#define	FIF_SDRPOWERMNG			(FIF_S1_ADDRESS + 0x1008)
#define	FIF_SDRCMDCTRL			(FIF_S1_ADDRESS + 0x1100)

/* FIF register value */
#define FIF_WRITE_THROUGH		0x00000033
#define FIF_WRITE_BUFFER_EMPTY	0x00000011

/* SMU registers */
#define	SMU_ADDRESS				0xC8010000
#define	SMU_PLLSEL				(SMU_ADDRESS + 0x0040)
#define	SMU_LPLL_CTRL2			(SMU_ADDRESS + 0x0054)
#define	SMU_HPLL_CTRL4			(SMU_ADDRESS + 0x0064)
#define	SMU_INT_RAW_STATUS		(SMU_ADDRESS + 0x0194)
#define	SMU_INT_CLEAR			(SMU_ADDRESS + 0x01A4)

/* SMU register value */
#define MASK_SMU_PLLSEL         0x00000003
#define SMU_PLLSEL_OSC          0x00000000
#define SMU_PLL_STANDBY         0x00000000
#define SMU_INT_MASK            0x00000001


/* 2007.05.15 fuj : Applied Deep-Power-Down */
#ifdef CONFIG_ICX1111

/*
 * warm boot handler (dummy routine)
 */
BEGIN_FUNC(warmboot_handler)
	.global warmboot_handler
	nop
END_FUNC(warmboot_handler)

#endif	/* CONFIG_ICX1111 */

/*
 *  flush_kern_cache_all()
 *
 *  Clean and invalidate the entire cache.
 */
BEGIN_FUNC(arm926_flush_kern_cache_all)
    mov r2, #VM_EXEC
    mov ip, #0
__flush_whole_cache:
#ifdef CONFIG_CPU_DCACHE_WRITETHROUGH
    mcr p15, 0, ip, c7, c6, 0       @ invalidate D cache
#else
1:  mrc p15, 0, r15, c7, c14, 3     @ test,clean,invalidate
    bne 1b
#endif
    tst r2, #VM_EXEC
    mcrne   p15, 0, ip, c7, c5, 0       @ invalidate I cache
    mcrne   p15, 0, ip, c7, c10, 4      @ drain WB
    mov pc, lr
END_FUNC(arm926_flush_kern_cache_all)


/*
 *  Variables
 */

cache_clean_ptr:
    .word	arm926_flush_kern_cache_all

/* register address */
reg_fif_wbufctrl:
    .word   FIF_WBUFCTRL
reg_fif_wbufstate:
    .word   FIF_WBUFSTATE
reg_smu_pll_sel:
    .word   SMU_PLLSEL
reg_smu_int_clear:
    .word   SMU_INT_CLEAR
reg_smu_int_raw_status:
    .word   SMU_INT_RAW_STATUS
reg_smu_lpll_ctrl2:
    .word   SMU_LPLL_CTRL2
reg_smu_hpll_ctrl4:
    .word   SMU_HPLL_CTRL4

/* register value */
val_write_through:
    .word   FIF_WRITE_THROUGH
val_write_buffer_empty:
    .word   FIF_WRITE_BUFFER_EMPTY

/* 2007.05.15 fuj : Applied Deep-Power-Down */
#ifdef CONFIG_ICX1111
reg_fif_sdrpowermng:
	.word	FIF_SDRPOWERMNG
reg_fif_sdrcmdctrl:
	.word	FIF_SDRCMDCTRL

val_sdrpowermng:
	.word	0x002E1100
val_cke_enable:
	.word	0x0000013A
val_sdrreqcompmask:
	.word   0x07000000
val_sdrreqcomp:
	.word   0x05000000
val_deep_power_down:
	.word   0x00000139
#endif	/* CONFIG_ICX1111 */

/*
 * mp200_change_osc()
 */
BEGIN_FUNC(mp200_change_osc)
	.global mp200_change_osc
    stmfd   sp!, {r0 - r12, lr}         @ save registers on stack

    @ clean data cache and invalidate WB
/* 2007.05.15 fuj : Applied Deep-Power-Down */
#ifdef CONFIG_ICX1111
	bl		arm926_flush_kern_cache_all
#else
    ldr     r1, cache_clean_ptr
    mov     lr, pc
    mov     pc, r1                      @ call mp200_pm_cache_clean()
#endif

    ldr     r4, reg_fif_wbufctrl
    ldr     r5, val_write_through
    str     r5, [r4]

    ldr     r4, reg_fif_wbufstate
    ldr     r5, val_write_buffer_empty
osc_buff_readwait:
    ldr     r6, [r4]
    cmp     r6, r5
    bne     osc_buff_readwait

    ldr     r4, reg_smu_int_clear       @outl(SMU_INT_MASK, SMU_INT_CLEAR);
    mov     r5, #SMU_INT_MASK
    str     r5, [r4]

    @outl((inl(SMU_PLLSEL) & ~MASK_SMU_PLLSEL) | SMU_PLLSEL_OSC, SMU_PLLSEL);
    ldr     r4, reg_smu_pll_sel
    ldr     r5, [r4]
    bic     r5, r5, #MASK_SMU_PLLSEL
    mov     r6, #SMU_PLLSEL_OSC
    orr     r5, r5, r6
    str     r5, [r4]

    @while (inl(SMU_INT_RAW_STATUS) != SMU_INT_MASK) {};
    ldr     r4, reg_smu_int_raw_status
osc_raw_wait:
    ldr     r5, [r4]
    cmp     r5, #SMU_INT_MASK
    bne     osc_raw_wait
#ifndef PM_PLL_OFF_DISABLE
    ldr     r6, reg_smu_lpll_ctrl2      @outl( SMU_PLL_STANDBY, SMU_LPLL_CTRL2 );
    mov     r5, #SMU_PLL_STANDBY
    str     r5, [r6]
    ldr     r6, reg_smu_hpll_ctrl4      @outl( SMU_PLL_STANDBY, SMU_HPLL_CTRL4 );
    mov     r5, #SMU_PLL_STANDBY
    str     r5, [r6]
#endif /* !PM_PLL_OFF_DISABLE */

    ldmfd   sp!, {r0 - r12, pc}         @ restore registers and return
END_FUNC(mp200_change_osc)

/* 2007.05.15 fuj : Applied Deep-Power-Down */
#ifdef CONFIG_ICX1111

/*
 * mp200_cpu_poweroff()
 */
BEGIN_FUNC(mp200_cpu_poweroff)
	.global mp200_cpu_poweroff

    @ clean data cache and invalidate WB
	bl		arm926_flush_kern_cache_all

    ldr     r4, reg_fif_wbufctrl
    ldr     r5, val_write_through
    str     r5, [r4]

    ldr     r4, reg_fif_wbufstate
    ldr     r5, val_write_buffer_empty
write_buffer_empty_loop2:
    ldr     r6, [r4]
    cmp     r6, r5
    bne     write_buffer_empty_loop2

    @
    ldr     r4, reg_fif_sdrpowermng
    ldr     r5, val_sdrpowermng
    str     r5, [r4]

    @ CKE enable
    ldr     r4, reg_fif_sdrcmdctrl
    ldr     r5, val_cke_enable
    str     r5, [r4]

    @ wat for completion
    ldr     r9, val_sdrreqcompmask
    ldr     r8, val_sdrreqcomp
wait_for_completion:
    ldr     r6, [r4]
    and     r6, r6, r9
    cmp     r6, r8
    bne     wait_for_completion

    @ deep power down
    ldr     r5, val_deep_power_down
    str     r5, [r4]

    @ wait for interrupt
    mcr     p15, 0, r0, c7, c0, 4
    mov     pc, lr
END_FUNC(mp200_cpu_poweroff)


/*
 * dummy function for getting warmboot_handler's size
 */
	.global warmboot_handler_size
warmboot_handler_size:
	.align;
	.word   . - warmboot_handler

#endif	/* CONFIG_ICX1111 */

#endif	/* CONFIG_ICX1087 */
