/*
 * bios_smu.h
 *
 * Copyright 2006,2007 Sony Corporation.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

#ifndef _BIOS_SMU_H_
#define _BIOS_SMU_H_

#include "bios_common.h"

/****************************************************/
/* address of register								*/
/****************************************************/

/* RESETERQ */
#define SMU_RESETREQ0           		0xC8010000
#define SMU_RESETREQ1A           		0xC8010004
#define	SMU_RESETREQ1B					0xC8010008
#define SMU_RESETREQ0_ENA  				0xC8010010
#define SMU_RESETREQ1A_ENA  			0xC8010014
#define	SMU_RESETREQ1B_ENA				0xC8010018

/* PLL */
#define	SMU_PLLSEL						0xC8010040
#define SMU_PLLSEL2                     0xC8010044
#define SMU_OSC_CTRL					0xC801004C
#define	SMU_LPLL_CTRL2					0xC8010054
#define	SMU_HPLL_CTRL4					0xC8010064

/* CLOCK GATE */
#define	SMU_GCLKCTRL0					0xC8010080
#define	SMU_GCLKCTRL1					0xC8010084
#define SMU_GCLKCTRL2       			0xC8010088
#define	SMU_GCLKCTRL0ENA				0xC801008C
#define	SMU_GCLKCTRL1ENA				0xC8010090
#define SMU_GCLKCTRL2ENA				0xC8010094
#define	SMU_AHBCLKCTRL0					0xC80100A0
#define	SMU_AHBCLKCTRL1					0xC80100A4
#define	SMU_APBCLKCTRL					0xC80100A8
#define	SMU_CLKCTRL						0xC80100AC

/* CLOCK SELECT */
#define	SMU_LCD_LCLK_SEL				0xC8010100
#define	SMU_TI0_TIN_SEL					0xC8010120
#define SMU_PWM_SCLK_SEL				0xC8010128
#define SMU_REFCLKO_SEL					0xC801012C

/* DIVISOR */
#define	SMU_LCD_LCLK_DIV				0xC8010130
#define	SMU_SPI0_SCLK_DIV				0xC8010140
#define SMU_MWI_SCLK_DIV        		0xC8010158
#define SMU_REFCLKO_DIV					0xC8010164

/* PCTL */
#define SMU_PCTL_SDR 				    0xC8010204
#define SMU_GIOCTL_PM1 				    0xC8010218
#define	SMU_PCTL_SPI					0xC801021C
#define SMU_PCTL_C2         			0xC8010220
#define	SMU_PULLCTL_SDR					0xC8010228
#define	SMU_PULLCTL_NAND				0xC801022C
#define SMU_PULLCTL_DSPDEBUG   			0xC8010238
#define SMU_PULLCTL_P2		   			0xC8010248

/* BOOT MODE */
#define	SMU_MODE_ST						0xC8010284

/* GPIO */
#define SMU_GPO             			0xC8010288


/****************************************************/
/* pll device code									*/
/****************************************************/
/* SEL */
#define	SMU_PLLSEL_OSC					0x00000000
#define	SMU_PLLSEL_RTC					0x00000001
#define	SMU_PLLSEL_PLL					0x00000002
#define SMU_PLL_STANDBY         		0x00000000
#define	MASK_SMU_PLLSEL					0x00000003
/* SEL2 */
#define	SMU_NRM_CK_SEL					0x00000004
/* OSC */
#define SMU_OSC_ENA_DISABLE     		0x00000004	/* 3ms wait */

/****************************************************/
/* reset device code								*/
/****************************************************/
#define SMU_RST_RESET           		0x00000000
/* RESETREQ0 */
#define	SMU_RESETDEVICE_PWM				0x00000010
#define	SMU_RESETDEVICE_LCD				0x00000001
#define	SMU_RESETDEVICE_TFD				0x00000100
#define	SMU_RESETDEVICE_TI0				0x00000002
/* RESETREQ1A */
#define	MASK_SMU_PE0_RST				0x00000001
/* RESETREQ1B */
#define	SMU_RESETDEVICE_SPI0			0x00000001

/****************************************************/
/* gatedclock code									*/
/****************************************************/
#define SMU_GCK_OFF         			0x00000000
#define SMU_GCK_ON          			0xffffffff
/* GCLKCTRL0 */
#define	SMU_CLOCKGATE_LCD_CLK			0x00000080
#define	SMU_CLOCKGATE_LCD_PCLK			0x00000100
#define	SMU_CLOCKGATE_LCD_LCLK			0x00000200
#define	SMU_CLOCKGATE_TFD_PCLK			0x00000400
#define	SMU_CLOCKGATE_TFD_LCLK			0x00000800
/* GCLKCTRL1 */
#define MASK_SMU_MWI_GCK        		0x30000000
#define	SMU_CLOCKGATE_TI0_TIN			0x00200000
#define	SMU_CLOCKGATE_TI0_PCLK			0x00100000
#define	SMU_CLOCKGATE_SPI0_PCLK			0x00000800
#define	SMU_CLOCKGATE_SPI0_SCLK			0x00001000
/* GCLKCTRL2 */
#define	SMU_CLOCKGATE_REFCLKO			0x00008000
#define	SMU_CLOCKGATE_PWM_SCLK			0x00000040
#define	SMU_CLOCKGATE_PWM_CLK			0x00000020

/****************************************************/
/* gatedclock automatic control code				*/
/****************************************************/
/* AHBCLKCTRL0 */
#define	SMU_CLKCTRL_LCD					0x00000010
/* AHBCLKCTRL1 */
#define	SMU_CLKCTRL_PB0					0x00000008
#define	SMU_CLKCTRL_PB1					0x00000010
/* APBCLKCTRL */
#define	SMU_CLKCTRL_TI0PCLK				0x00010000
#define	SMU_CLKCTRL_LCDPCLK				0x00000004
#define	SMU_CLKCTRL_TFDPCLK				0x00000008
/* CLKCTRL */
#define	SMU_CLKCTRL_SPI0PCLK			0x00004000
#define	SMU_CLKCTRL_SPI0SCLK			0x00000002

/****************************************************/
/* clock select code								*/
/****************************************************/
/* PWM */
#define SMU_PWM_SCLK_SEL_OSC			0x00000000
/* TI0 */
#define	SMU_TIN_SEL_32K					0x00000004
/* REFCLK */
#define SMU_REFCLKO_SEL_OSC				0x00000000
#define SMU_REFCLKO_DIV_DEF				0x00000000
/* MWI */
#define SMU_MWI_DIV_OSC_VAL     		0x00000003

/****************************************************/
/* gio control code									*/
/****************************************************/
/* PM1 */
#define	SMU_GIOCTL_PM1_GPO				0x00000001

/****************************************************/
/* pull control code								*/
/****************************************************/
/* SDR */
#define SMU_PCTL_SDR_FLOATING       	0x00000000
#define SMU_PULLCTL_SDR_PDOWN       	0x0007ffff
/* OTHERS */
/* 2006.11.24 fuj : modify pull control value */
/* #define	SMU_PCTL_C2_PWM					0x00000080 */
#define	SMU_PCTL_C2_PWM					0x00000080
#define	SMU_PULLCTL_BLTCTL				0x00080008
/* 2006.11.24 fuj : modify pull control value */
/* #define	SMU_PULLCTL_P2_PWM0				0x00000080 */
#define	SMU_PULLCTL_P2_PWM0				0x00000000
/* NAND(CS_SEL) */
#define	SMU_PULLCTL_NAND_XWP			0x00000080
#define	SMU_PULLCTL_CS_SEL				0x00000010

/****************************************************/
/* gpio code										*/
/****************************************************/
#define	SMU_GPO5_ENABLE					0x00200020
#define	SMU_GPO5_DISABLE				0x00200000

/****************************************************/
/* boot mode code									*/
/****************************************************/
#define	SMU_MODE_ST_MASK				0x0000001F
#define	SMU_MODE_ST_MS					0x00000006

#endif
