/*
 * bios_spi.h
 *
 * Copyright 2006,2007 Sony Corporation.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */


#ifndef _BIOS_SPI_H_
#define _BIOS_SPI_H_

#include "bios_common.h"


/*---------------------------- user setting --------------------------*/

/* SPI0/1 */
#define SPI_SPI0	0
#define SPI_SPI1	1
#define SPI_MAX_NUM	2

/* CS 0/1/2/3 */
#define SPI_CS0		0
#define SPI_CS1		1
#define SPI_CS2		2
/* 2007.03.06 fuj : Applied various models */
#ifdef CONFIG_ICX1111
#define SPI_CS3		3
#endif
#define MAX_CS		3

/* NB */
#define SPI_BIT8	0
#define SPI_BIT16	1
#define SPI_BIT24	2
#define SPI_BIT32	3

/* clock */
#define SPI_SCLOCK_1500KHz	(1500000)
#define SPI_SCLOCK_3MHz		(3000000)
#define SPI_SCLOCK_6MHz		(6000000)
#define SPI_SCLOCK_12MHz	(12000000)

/* mode */
#define SPI_MODE_MS_M        (0 << 1)
#define SPI_MODE_MS_S        (1 << 1)

/* Pol */
							/* CSW:8clock	*/
/* 2007.03.06 fuj : Applied various models */
#ifdef CONFIG_ICX1111
							/* CK0_DLY:off, CK0_POL:neg, CS0_POL:neg  */
							/* CK2_DLY:off, CK2_POL:neg, CS2_POL:neg  */
#define SPI0_POL_SET		0x000070C3
#else
							/* CK2_DLY:off, CK2_POL:neg, CS2_POL:neg  */
#define SPI0_POL_SET		0x000080C0
#endif

/* Tx_TIM */
#define SPI_TX_TIM_30WORD    0
#define SPI_TX_TIM_16WORD    1
#define SPI_TX_TIM_8WORD     2
#define SPI_TX_TIM_1WORD     3


/* structure */
typedef struct {
	UINT8	spi_num;
	UINT8	cs_num;
	UINT8	bit_num;
	UINT32	clock;
	UINT32	mode[MAX_CS];
	UINT32	pol;
	UINT32	tx_tim[MAX_CS];
	UINT32	counter;
}spi_config_t;

/* prototype */
extern BIOS_RESULT spiInit(spi_config_t *);
extern BIOS_RESULT spiEnd(void);
extern BIOS_RESULT spiWrite(spi_config_t *, UINT32 *);


/*---------------------------- SPI register --------------------------*/

/* register address */
#define MP_SP0_MODE			0x40130000
#define MP_SP0_POL			0x40130004
#define MP_SP0_CONTROL		0x40130008
#define MP_SP0_TX_TIM		0x4013000C
#define MP_SP0_TX_DATA		0x40130010
#define MP_SP0_RX_DATA		0x40130014
#define MP_SP0_STATUS		0x40130018
#define MP_SP0_RAW_STATUS	0x4013001C
#define MP_SP0_ENSET		0x40130020
#define MP_SP0_ENCLR		0x40130024
#define MP_SP0_FFCLR		0x40130028
#define MP_SP0_CONTROL2		0x40130034

/* setting value */

/* control */
#define CONTROL_TX_FULL  	(0x01 << 7)
#define CONTROL_RX_EMP   	(0x01 << 6)
#define CONTROL_RST      	(0x01 << 4)
#define CONTROL_WRT      	(0x01 << 3)
#define CONTROL_RD       	(0x01 << 2)
#define CONTROL_STOP     	(0x01 << 1)
#define CONTROL_START    	(0x01 << 0)

/* interrupt status */
#define SPI_ENCLR_ALLERR     0x0000007F /* (TERR|RDV|END|TX_UDR|RX_OVR) */
#define SPI_FFCLR_ALLERR     0x0000007F /* (TERR|RDV|END|TX_UDR|RX_OVR) */

/* NB */
#define SPI_NB_BIT8		(8 - 1)
#define SPI_NB_BIT16	(16 - 1)
#define SPI_NB_BIT24	(24 - 1)
#define SPI_NB_BIT32	(32 - 1)

#define SPI_MAX_WAIT		64				/* poling count			  */

#endif    /* _BIOS_SPI_ */

