/*
 * icx1087_beep.c
 *
 * Copyright 2006 Sony Corporation.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */


#include <common.h>

#ifdef CONFIG_ICX1087

#include "bios_spi.h"
#include "bios_smu.h"
#include "bios_gio.h"
#include "icx1087.h"


#define	SPI_DATA(r, d)	(UINT32)((0x94<<16) | (r<<8) | d)

/** SPI configuration **/
static spi_config_t spi_config = {
	SPI_SPI0,										/* SPI0				*/
	SPI_CS2,										/* CS2				*/
	SPI_BIT24,										/* 24bit			*/
	SPI_SCLOCK_6MHz,								/* 6MHz				*/
	{SPI_MODE_MS_M, SPI_MODE_MS_M, SPI_MODE_MS_M},	/* all Master		*/
	SPI0_POL_SET,									/* see. bios_spi.h	*/
	{SPI_TX_TIM_1WORD, SPI_TX_TIM_1WORD, SPI_TX_TIM_1WORD},	/* 1word	*/
	1												/* counter			*/
};

/**
 * Stop to sound a beep
 *   Input  : none
 *   Output : none
 */
void icx1087_off_beep(void)
{
	UINT32 data;

	/* stop to sound a beep */
	data = SPI_DATA(0x14, 0x00);	/* REG14: Beep:disable					*/
	spiWrite(&spi_config, &data);

	/* DAC mute on */
	data = SPI_DATA(0x08, 0x83);	/* REG08: DAC Mute:enable				*/
	spiWrite(&spi_config, &data);

	return;
}

/**
 * Sound a beep
 *   Called from board_init(mp200.c)
 *   Input  : tim	: period to sound a beep(us)
 *                    (0:coutinuous)
 *   Output : none
 */
void icx1087_on_beep( UINT32 tim )
{
	UINT32 data;

	/* DAC mute off */
	data = SPI_DATA(0x08, 0x80);	/* REG08: DAC Mute:disable				*/
	spiWrite(&spi_config, &data);

	/* sound a beep */
	data = SPI_DATA(0x14, 0xC0);	/* REG14: Beep:continuous				*/
	spiWrite(&spi_config, &data);
	if( tim > 0 ) {
		udelay(tim);
		icx1087_off_beep();
	}
	return;
}

/**
 * Initialize beep module
 *   Called from board_init(mp200.c)
 *   Input  : none
 *   Output : none
 */
void icx1087_init_beep(void)
{
	UINT32 data;

	udelay(40000);
	/* set Mute/HP */
/* 2006.11.24 fuj : set GPPUPD */
	gioPwc_write(PWC_GPIO_GPPUPD3, PWC_GPIO_GPPUPD3_HP);
	gioPwc_write(PWC_GPIO_GPPUPD4, PWC_GPIO_GPPUPD4_MUTE);
	gioPwc_ioout(PWC_GPIO_IOOUT2,
				 PWC_GPIO2_MUTE|PWC_GPIO2_HP, 0);
	if (icx1087_lsiver < PWC_LSIVER_ES12)
		gioPwc_ioout(PWC_GPIO_IOOUT2, PWC_GPIO2_HP, 1);
	gioPwc_iosel(PWC_GPIO_IOSEL2,
				 PWC_GPIO2_MUTE|PWC_GPIO2_HP, 1);

	/* setting GPIO */
	BIOS_MCR_OUTPW(SMU_GIOCTL_PM1, SMU_GIOCTL_PM1_GPO);
	BIOS_MCR_OUTPW(SMU_GPO, SMU_GPO5_DISABLE);

	/* cancel reset */
	BIOS_MCR_OUTPW(SMU_GPO, SMU_GPO5_ENABLE);

	/* initialize SPI */
	spiInit(&spi_config);

	/* set mode of CS42L51 to S/W mode */
	data = SPI_DATA(0x02, 0x7F);
	spiWrite(&spi_config, &data);

	/* setting CS42L51 */
	data = SPI_DATA(0x00, 0x99);	/* REG00: select hidden register?	 	*/
	spiWrite(&spi_config, &data);
	data = SPI_DATA(0x37, 0x05);	/* REG37: amount of current				*/
	spiWrite(&spi_config, &data);

	data = SPI_DATA(0x00, 0x99);	/* REG00: select hidden register?	 	*/
	spiWrite(&spi_config, &data);
	data = SPI_DATA(0x38, 0xC0);	/* REG38: DAC/HP AMP Queiscent power saving	*/
	spiWrite(&spi_config, &data);

	data = SPI_DATA(0x03, 0x2F);	/* REG03: Single Speed Mode, MCLK/2 	*/
	spiWrite(&spi_config, &data);
	data = SPI_DATA(0x04, 0x40);	/* REG04: Master, Left Justified		*/
	spiWrite(&spi_config, &data);
	data = SPI_DATA(0x08, 0x80);	/* REG08: HP_GAIN:0.7099, DAC Mute:disable	*/
	spiWrite(&spi_config, &data);
	data = SPI_DATA(0x09, 0x42);	/* REG09: SPE -> DAC, Soft Ramp			*/
	spiWrite(&spi_config, &data);
	
	data = SPI_DATA(0x12, 0xF0);	/* REG12: Beep Freq:2kHz				*/
	spiWrite(&spi_config, &data);
	data = SPI_DATA(0x13, ((-19)&0x1F));/* REG13: Beep Vol:-38dB			*/
	spiWrite(&spi_config, &data);
	data = SPI_DATA(0x14, 0x00);	/* REG14: Beep:disable					*/
	spiWrite(&spi_config, &data);

	data = SPI_DATA(0x16, 0x00);	/* REG16: AOUTA Vol:0dB					*/
	spiWrite(&spi_config, &data);
	data = SPI_DATA(0x17, 0x00);	/* REG16: AOUTB Vol:0dB					*/
	spiWrite(&spi_config, &data);

	data = SPI_DATA(0x21, 0x80);	/* REG21: Charge Pump Frequency			*/
	spiWrite(&spi_config, &data);

	data = SPI_DATA(0x02, 0x1E);	/* REG02: Powerup CS42L51				*/
	spiWrite(&spi_config, &data);

	/* set clock */
	BIOS_MCR_OUTPW(SMU_REFCLKO_SEL, SMU_REFCLKO_SEL_OSC);
	BIOS_MCR_OUTPW(SMU_REFCLKO_DIV, SMU_REFCLKO_DIV_DEF);
	BIOS_MCR_OUTPW(SMU_GCLKCTRL2ENA, SMU_CLOCKGATE_REFCLKO);
	BIOS_MCR_OUTPW(SMU_GCLKCTRL2, SMU_CLOCKGATE_REFCLKO);
	BIOS_MCR_OUTPW(SMU_GCLKCTRL2ENA, 0);

	/* wait 70ms */
	udelay(70000);

	/* change to HP */
	if (icx1087_lsiver >= PWC_LSIVER_ES12)
		gioPwc_ioout(PWC_GPIO_IOOUT2, PWC_GPIO2_HP, 1);
	else
		gioPwc_ioout(PWC_GPIO_IOOUT2, PWC_GPIO2_HP, 0);

	/* wait 40ms */
	udelay(40000);

	/* mute off */
	gioPwc_ioout(PWC_GPIO_IOOUT2, PWC_GPIO2_MUTE, 1);
	udelay(5000);
}

/**
 * Disable beep
 *   Input  : none
 *   Output : none
 */
void icx1087_disable_beep(void)
{
	icx1087_off_beep();

	spiEnd();

	return;
}

/**
 * Exit beep
 *   Input  : none
 *   Output : none
 */
void icx1087_exit_beep(void)
{
	/* mute on */
	gioPwc_ioout(PWC_GPIO_IOOUT2, PWC_GPIO2_MUTE, 0);
	udelay(14000);

	/* change to HP */
	gioPwc_ioout(PWC_GPIO_IOOUT2, PWC_GPIO2_HP, 0);

	/* setting GPIO */
	BIOS_MCR_OUTPW(SMU_GPO, SMU_GPO5_DISABLE);

	/* wait 40ms */
	udelay(40000);

	return;
}

#endif /* CONFIG_ICX1087 */


