/*
 * icx1087_key.c
 *
 * Copyright 2006,2007 Sony Corporation.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

#include <common.h>

#ifdef CONFIG_ICX1087

#include "bios_gio.h"
#include "icx1087.h"


/**********/
/* Macros */
/**********/
#define KEY_HOLD_INTERVAL   (10000)         /* 10ms		*/
#define KEY_HOLD_COUNT      (500)           /* wait 5s	*/

#define KEY_HOLD_WAIT_CHAT  (30000)         /* anti-chattering 30ms	*/


/********************/
/* Global variables */
/********************/
UINT16 icx1087_key_data = KEY_PAD_DFLT;		/* AD value of keys	*/
											/*  bit15 - 8 : AD0	*/
											/*  bit 7 - 0 : AD1	*/

const UINT8 icx1087_key_ad_tbl[] = {
	29, 61, 93, 125, 158, 192, 224, 249
};


/*******************/
/* Global routines */
/*******************/

/**
 * Get HOLD 
 *   Called from display_opening(mp200.c)
 *   Input  : none
 *   Output : 1:HOLD ON / 0:HOLD OFF
 */
UINT8 icx1087_get_hold(void)
{
    UINT8 stat1, stat2;

   	/* set GPIO input */
   	gioPwc_iosel(PWC_GPIO_IOSEL1, PWC_GPIO1_HOLD, 0);
   	/* read HOLD key */
   	stat1 = gioPwc_read(PWC_GPIO_IOIN1) & PWC_GPIO1_HOLD;

/* 2006.12.7 fuj : anti-chattering */
	while (1) {
		udelay(KEY_HOLD_WAIT_CHAT);

    	/* read HOLD key */
    	stat2 = gioPwc_read(PWC_GPIO_IOIN1) & PWC_GPIO1_HOLD;

		if (stat1 == stat2)
			break;
		
		stat1 = stat2;
	}

	return (stat1) ? 0 : 1;
}

/**
 * Wait HOLD off
 *   Called from display_opening(mp200.c)
 *   Input  : none
 *   Output : 1:to power off / 0:boot up
 */
UINT8 icx1087_chk_hold(void)
{
    UINT8 stat;
    UINT32 cnt;
/* 2007.05.17 fuj : Applied RTC wake-up */
#ifdef CONFIG_ICX1111
	UINT8 rtc_flag = 1;
#endif

#ifdef CONFIG_DET_ADP
/* 2007.05.17 fuj : Applied RTC wake-up */
#ifdef CONFIG_ICX1111
	if (icx1087_det_psw(ICX1087_PSW_ADP) || icx1087_det_psw(ICX1087_PSW_USB))
		return ICX11xx_CANCEL_HOLD_PSW;
	else if (icx11xx_chk_rtc()) {
	   	if (icx11xx_chk_rtc_date(1))
			return ICX11xx_CANCEL_HOLD_RTC;
		else
			rtc_flag = 0;
	}
#else
	if (icx1087_det_psw(ICX1087_PSW_ADP) || icx1087_det_psw(ICX1087_PSW_USB))
		return 0;
#endif
#else
/* 2007.05.17 fuj : Applied RTC wake-up */
#ifdef CONFIG_ICX1111
	if (icx1087_det_psw(ICX1087_PSW_USB))
		return ICX11xx_CANCEL_HOLD_PSW;
	else if (icx11xx_chk_rtc()) {
		if (icx11xx_chk_rtc_date(1))
			return ICX11xx_CANCEL_HOLD_RTC;
		else
			rtc_flag = 0;
	}
#else
	if (icx1087_det_psw(ICX1087_PSW_USB))
		return 0;
#endif
#endif

    /* set GPIO input */
    gioPwc_iosel(PWC_GPIO_IOSEL1, PWC_GPIO1_HOLD, 0);

    /* read HOLD key */
    stat = (gioPwc_read(PWC_GPIO_IOIN1) & PWC_GPIO1_HOLD) ? 0 : 1;

	if (stat) { 	/* HOLD ON : stat = 1 */
		/* display HOLD */
        icx1087_disp_lcd(LCD_IMG_HOLD, 1);

        /* wait for Hold off or elapse 5s */
        for (cnt = KEY_HOLD_COUNT; cnt > 0; cnt--) {
            udelay(KEY_HOLD_INTERVAL);
			/* check HOLD */
            stat = (gioPwc_read(PWC_GPIO_IOIN1) & PWC_GPIO1_HOLD) ? 0 : 1;
/* 2007.05.17 fuj : Applied RTC wake-up */
#ifdef CONFIG_ICX1111
			if (!stat) {
				stat = ICX11xx_CANCEL_HOLD_OFF;
				break;
			}
#else
            if (!stat)
                break;
#endif

			/* check ADP and VBUS */
#ifdef CONFIG_DET_ADP
/* 2007.05.17 fuj : Applied RTC wake-up */
#ifdef CONFIG_ICX1111
			if (icx1087_det_psw(ICX1087_PSW_ADP)
			 || icx1087_det_psw(ICX1087_PSW_USB)) {
				stat = ICX11xx_CANCEL_HOLD_PSW;
				break;
			}
			else if (rtc_flag && icx11xx_chk_rtc()) {
				if (icx11xx_chk_rtc_date(1)) {
					stat = ICX11xx_CANCEL_HOLD_RTC;
					break;
				}
				else
					rtc_flag = 0;
			}
#else
			if (icx1087_det_psw(ICX1087_PSW_ADP)
			 || icx1087_det_psw(ICX1087_PSW_USB)) {
				stat = 0;
				break;
			}
#endif
#else
/* 2007.05.17 fuj : Applied RTC wake-up */
#ifdef CONFIG_ICX1111
			if (icx1087_det_psw(ICX1087_PSW_USB)) {
				stat = ICX11xx_CANCEL_HOLD_PSW;
				break;
			}
			else if (rtc_flag && icx11xx_chk_rtc()) {
			   	if (icx11xx_chk_rtc_date(1)) {
					stat = ICX11xx_CANCEL_HOLD_RTC;
					break;
				}
				else
					rtc_flag = 0;
			}
#else
			if (icx1087_det_psw(ICX1087_PSW_USB)) {
				stat = 0;
				break;
			}
#endif
#endif
			/* whether keypad is pressed */
			icx1087_get_keypad();
			if (ICX1087_KEY_PRESSED())
				cnt = KEY_HOLD_COUNT + 1;
        }
/* 2007.05.17 fuj : Applied RTC wake-up */
#ifdef CONFIG_ICX1111
		if (cnt <= 0)
			stat = ICX11xx_CANCEL_HOLD_TO;
#endif

		/* set Keypad to default(released) */
		icx1087_key_data = KEY_PAD_DFLT;
		icx1087_set_keypad();

        icx1087_off_lcd();
/* 2007.05.30 fuj : Applied RTC wake-up */
#ifdef CONFIG_ICX1111
		if (stat == ICX11xx_CANCEL_HOLD_RTC)
			icx11xx_exit_lcd();
#endif
    }
/* 2007.05.17 fuj : Applied RTC wake-up */
#ifdef CONFIG_ICX1111
	else {
		stat = ICX11xx_CANCEL_HOLD_OFF;
	}
#endif

    return stat;
}

/**
 * Read Keypad (save the value in global variable)
 *   Called from display_opening(mp200.c)
 *   Input  : none
 *   Output : none
 */
void icx1087_get_keypad(void)
{
	UINT8 ad_data;

	/* get A/D value */
	gioPwc_init_ad();
	icx1087_key_data &= ~KEY_PAD_MASK;
	if( gioPwc_read_ad(PWC_AD_CH_KEY0, &ad_data) )	/* read Key0	*/
		icx1087_key_data |= (UINT32)ad_data << 8;
	else
		icx1087_key_data |= KEY_AD_MASK << 8;

	if( gioPwc_read_ad(PWC_AD_CH_KEY1, &ad_data) )	/* read Key1	*/
		icx1087_key_data |= (UINT32)ad_data;
	else
		icx1087_key_data |= KEY_AD_MASK;
	gioPwc_exit_ad();

	/* set Key data to SRAM */
	icx1087_set_keypad();
}

/**
 * Set Keypad code to SRAM in order to notify it Linux
 *   Called from display_opening(mp200.c)
 *   Input  : none
 *   Output : none
 */
void icx1087_set_keypad(void)
{
	UINT32 kdata;

	kdata = KEY_MAGIC_NUM | (icx1087_key_data & KEY_PAD_MASK);
	BIOS_MCR_OUTPW(KEY_SRAM_ADR, kdata);
}

#endif /* CONFIG_ICX1087 */

