/*
 * nand_local.h
 *
 * Copyright 2006,2007 Sony Corporation.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

#ifndef __NAND_LOCAL_H
#define __NAND_LOCAL_H

/* MP201 NAND registers */

/* base address */
#define MP201_BANK4_BASE	0x40000000
#define MP201_NAND_OFFSET	0x00170000
#define MP201_NAND_BASE		(MP201_BANK4_BASE + MP201_NAND_OFFSET)

/* BBC register */
#define NAND_ATBAR		(MP201_NAND_BASE + 0x0000)
#define NAND_ATCR		(MP201_NAND_BASE + 0x0004)
#define NAND_MSR		(MP201_NAND_BASE + 0x000c)
#define NAND_INTRAWSTATUS	(MP201_NAND_BASE + 0x0014)
#define NAND_INTFFCLR		(MP201_NAND_BASE + 0x0020)
#define NAND_FRD		(MP201_NAND_BASE + 0x0024)

/* EMMU register */
#define NAND_CR			(MP201_NAND_BASE + 0x0200)
#define NAND_EICR		(MP201_NAND_BASE + 0x0208)
#define NAND_CECR		(MP201_NAND_BASE + 0x0214)
#define NAND_CDR		(MP201_NAND_BASE + 0x0218)
#define NAND_CWR		(MP201_NAND_BASE + 0x0220)
#define NAND_AWR		(MP201_NAND_BASE + 0x0224)
#define NAND_LAWR		(MP201_NAND_BASE + 0x0228)
#define NAND_BDRR		(MP201_NAND_BASE + 0x0238)
#define NAND_FPSCR		(MP201_NAND_BASE + 0x023C)
#define NAND_FWSR		(MP201_NAND_BASE + 0x0240)
#define NAND_RSSR		(MP201_NAND_BASE + 0x02a0)
#define NAND_RSEVxR		(MP201_NAND_BASE + 0x02c0)
#define NAND_RSEPxR		(MP201_NAND_BASE + 0x02d0)
#define NAND_SDHR		(MP201_NAND_BASE + 0x0300)
#define NAND_SDH0R		(MP201_NAND_BASE + 0x0300)
#define NAND_SDH1R		(MP201_NAND_BASE + 0x0304)
#define NAND_SDH2R		(MP201_NAND_BASE + 0x0308)
#define NAND_SDH3R		(MP201_NAND_BASE + 0x030C)
#define NAND_SDH4R		(MP201_NAND_BASE + 0x0310)
#define NAND_SDH5R		(MP201_NAND_BASE + 0x0314)
#define NAND_SDH6R		(MP201_NAND_BASE + 0x0318)
#define NAND_SDH7R		(MP201_NAND_BASE + 0x031C)
/* PSC registers */
#define NAND_PSCR		(MP201_NAND_BASE + 0x0400)
#define NAND_IPR		(MP201_NAND_BASE + 0x0404)
#define NAND_ITR		(MP201_NAND_BASE + 0x0420)

/* LDMA registers */
#define NAND_LIRSR		(MP201_NAND_BASE + 0x0610)
#define NAND_LICR		(MP201_NAND_BASE + 0x0614)

/* BBC unit value */
#define BBC_INT_512INT		(0x01)
#define BBC_INT_PAGEINT		(0x02)
#define BBC_INT_MASK		(0x03)

#define NAND_MSR_FBDC		(0x1 << 31)

#define NAND_MSR_PSIZE_512	(0x0 << 6)
#define NAND_MSR_PSIZE_2048	(0x2 << 6)
#define NAND_MSR_PSIZE_4096	(0x3 << 6)
#define NAND_MSR_PSIZE_MASK	(0x3 << 6)

#define NAND_MSR_FTYPE_8	(0x0 << 4)
#define NAND_MSR_FTYPE_16	(0x1 << 4)
#define NAND_MSR_FTYPE_MASK	(0x3 << 4)

#define NAND_MSR_AHBDIR		(0x1 << 1)
#define NAND_MSR_AHBMEN		(0x1)

#define NAND_FRD_CACT		(0x1 << 19)
#define NAND_FRD_DATA_MASK	0xffff

/* EMMU unit value */
#define NAND_CECR_CEZ		0x01
#define NAND_CECR_MASK		0x0f
#define NAND_RSSR_CALC_END	0x08
#define NAND_RSSR_UNCOR		0x04
#define NAND_RSSR_ERR		0x02
#define NAND_RSSR_NO_ERR	0x01
#define NAND_RSSR_MASK		0x0f

#define RS_UNCORRECT_ERROR	(NAND_RSSR_CALC_END | NAND_RSSR_UNCOR | NAND_RSSR_ERR)
#define RS_NO_ERROR		(NAND_RSSR_CALC_END | NAND_RSSR_NO_ERR)
#define RS_CORECT_ERROR		(NAND_RSSR_CALC_END | NAND_RSSR_ERR)

#define RS_CHECK_SIZE		512

/* PSC commands */
#define PSC_CMD_CESEL		0x0a
#define PSC_CMD_OUTCMD		0x10
#define PSC_CMD_OUTADRS		0x12
#define PSC_CMD_OUTLTADRS	0x14
#define PSC_CMD_CHKST		0x1e
#define PSC_CMD_DMAST		0xc0
#define PSC_CMD_INT		0xd0
#define PSC_CMD_STOPINT		0xd1
#define PSC_CMD_BTS		0xe0
#define PSC_CMD_WAITFOR		0xf0
#define PSC_CMD_WAITTC		0xf1
#define PSC_CMD_WAITBUSY	0xf2
#define PSC_CMD_WAITRDY		0xf3

#define PSC_START		0x01
#define PSC_CE0_ENA		0x0e
#define PSC_RYBYZ0_AND		0x10
#define PSC_RYBYZ0		0x01
#define PSC_DMA_START		0x05
#define PSC_DMA_RSOK		0x02
#define PSC_DMA_            0x01

#define LDMA_INT_RSERR		0x80
#define LDMA_INT_RSOK		0x20
#define LDMA_INT_PSC		0x08
#define LDMA_INT_LDMACTC	0x04
#define LDMA_INT_MASK		0xac

#define PSC_IIDR_ADR_SEND	0x01
#define PSC_IIDR_DMA_END	0x02

#define PSC_CMD_SET_BEGIN() \
{ \
	pointer = NAND_ITR; \
	psc_step = 0; \
}
#define PSC_CMD_SET(inst, ope) \
do { \
	outl(((((ope) << 8) | (inst))), (pointer)); \
	pointer += 4; \
	psc_step++; \
} while(0)
#define PSC_CMD_SET_END() outl(0, NAND_IPR)

#define PSC_SEQUENCE_END	(psc_step - 1)

/* standard NAND flash commands */
#define NAND_CMD_READ0		0x00
#define NAND_CMD_STATUS		0x70
#define NAND_CMD_READID		0x90
#define NAND_CMD_RESET		0xff
#define NAND_CMD_ERASE0		0x60
#define NAND_CMD_ERASE1		0xd0
#define NAND_CMD_DATAIN		0x80
#define NAND_CMD_PROGRAM	0x10
#define NAND_CMD_EDO		0xB1

/* extended commands for large page devices */
#define NAND_CMD_READSTART	0x30

#define NAND_DISABLE_CE(x) \
do { \
	unsigned int cez;\
	cez = (inl(NAND_CECR) & NAND_CECR_MASK); \
	outl((cez | (NAND_CECR_CEZ << (x))), (NAND_CECR)); \
} while(0)
#define NAND_ENABLE_CE(x) \
do { \
	unsigned int cez;\
	cez = (inl(NAND_CECR) & NAND_CECR_MASK); \
	outl((cez & ~(NAND_CECR_CEZ << (x))), (NAND_CECR)); \
} while(0)


/* NAND chip status bits */
#define NAND_STATUS_READY	0x40

/* buswitdh is 16 bit */
#define NAND_BUSWIDTH8		0x0
#define NAND_BUSWIDTH16		0x01

#define UNCORRECT_FLAG		(1 << 16)
#define NOERROR_FLAG		(1 << 24)

/* correct information */
struct correct_info {
	unsigned long addr;
	unsigned long val;
};

/* correct save data */
struct correct_data {
	struct correct_info info[4];
	unsigned long flag;
};

#endif /* __NAND_LOCAL_H */
