/*
 * cmd_icx1087_nand.c
 *
 * Copyright 2006,2007 Sony Corporation.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

#include <common.h>
#include <command.h>

#ifdef CONFIG_ICX1087
#if (CONFIG_COMMANDS & CFG_CMD_NANDBOOT)

#include "../board/mp201xx/icx1087/icx1087.h"
#include "../board/mp201xx/icx1087/bios_nand.h"

#define	NAND_PAGE_NUM	128
/* 2007.04.17 fuj */
#ifdef CONFIG_ICX1111
#define	NAND_PAGE_SIZE	4096
#else
#define	NAND_PAGE_SIZE	2048
#endif
#define	NAND_BLOCK_NUM	8192
#define	NAND_BLOCK_SIZE	(NAND_PAGE_NUM * NAND_PAGE_SIZE)

int do_nandboot ( cmd_tbl_t *cmdtp, int flag, int argc, char *argv[] )
{
	nand_boot_linux(ICX1087_BOOT_NRML);		/* never return */
	return 0;
}

U_BOOT_CMD(
    nandboot,  1,  0,  do_nandboot,
    "nandboot- Boot Linux from NAND\n",
    "\n"
);

int do_fwupboot ( cmd_tbl_t *cmdtp, int flag, int argc, char *argv[] )
{
	nand_boot_linux(ICX1087_BOOT_EMRG);		/* never return */
	return 0;
}

U_BOOT_CMD(
    fwupboot,  1,  0,  do_fwupboot,
    "fwupboot- Boot Linux from NAND in emergency F/W update mode\n",
    "\n"
);

static int my_atoi(char *str, int *rn, int base)
{
	int i;
	int n = strlen(str);

	*rn = 0;
	for (i=0; i<n; i++) {
		*rn *= base;
		if (('0' <= str[i]) && (str[i] <= '9')) {
			*rn += (int)str[i] - 0x30; 
		}
		else if (base == 16) {
			if (('a' <= str[i]) && (str[i] <= 'f'))
				*rn += (int)str[i] - 0x61 + 10; 
			else if (('A' <= str[i]) && (str[i] <= 'F'))
				*rn += (int)str[i] - 0x41 + 10; 
			else
				return -1;
			
		}
		else {
			return -1;
		}
	}
	return 0;
}

int do_nand_display ( cmd_tbl_t *cmdtp, int flag, int argc, char *argv[] )
{
	int b_no = -1;
	int p_no = -1;
	int len = -1;
	int adr;
	int id_flag = 0;
	int num, i, j;
	int rc;
	unsigned char idbuf[8], extbuf[6];
	unsigned char buf[NAND_PAGE_SIZE];

/* 2007.04.17 fuj */
#ifdef CONFIG_ICX1111
	unsigned int page_size;

	nandInitialize(NULL, idbuf);

	for (i=0; i<8; i++) idbuf[i] = 0;
	rc = nandReadId(0, idbuf);

	if((idbuf[3] & 0x03) == 0x01)
		page_size = 2048;
	else
		page_size = 4096;
#else
	nandInitialize(NULL);
#endif

	if (argc < 2) {
        /* invalid argument */
		printf ("Usage:\n%s\n", cmdtp->usage);
		return 1;
	}

	/* get parameters */
	for (i=1; i<argc; i++) {
		if (strcmp(argv[i], "-i") == 0) {
			id_flag = 1;
		}
		else {
        	if (my_atoi(argv[i], &num, 10) == 0) {
				if (b_no < 0)
					b_no = num;
				else if (p_no < 0)
					p_no = num;
				else if (len < 0)
					len = num;
			}
		}
	}

	if (len < 0) len = 1;
	/* display ID */
	if (id_flag) {
/* 2007.04.17 fuj */
#ifndef CONFIG_ICX1111
		for (i=0; i<8; i++) idbuf[i] = 0;
		rc = nandReadId(0, idbuf);
#endif
		printf(" NAND ID:");
		if (rc == 0) {
			for (i=0; i<8; i++)
				printf(" %02X", (uint)idbuf[i]);
			printf("\n");
		}
		else
			printf(" read error(%d)\n", rc);
	}

	/* check parameters */
	if ((b_no < 0) || (p_no < 0) ||
		(b_no > NAND_BLOCK_NUM) || (p_no > NAND_PAGE_NUM) ||
		(len > (NAND_PAGE_NUM * NAND_BLOCK_NUM))){
        /* invalid argument */
		if (id_flag == 0)
			printf ("Usage:\n%s\n", cmdtp->usage);
		return 1;
	}

	/* display dump */
	adr = b_no * NAND_PAGE_NUM + p_no;
	rc = nandReadPage(0, adr, buf, extbuf);
/* 2007.04.17 fuj */
#ifdef CONFIG_ICX1111
	printf(" NAND Read Block[%d] Page[%d]->address[%08X] :",
			b_no, p_no, adr*page_size);
#else
	printf(" NAND Read Block[%d] Page[%d]->address[%08X] :",
			b_no, p_no, adr*NAND_PAGE_SIZE);
#endif
	if(rc == 0) {
		printf("\n");
/* 2007.04.17 fuj */
#ifdef CONFIG_ICX1111
		for(i=0; i<page_size; i+=16) {
#else
		for(i=0; i<NAND_PAGE_SIZE; i+=16) {
#endif
			printf("  %08X:", i);
			for(j=0; j<16; j++) {
				printf(" %02X", buf[i+j]);
			}
			printf("    ");
			for(j=0; j<16; j++) {
				if ((buf[i+j] < 0x20) || (buf[i+j] > 0x7e))
	                putc ('.');
				else
					printf("%c", buf[i+j]);
			}
			printf("\n");
		}
		printf("  Redundancy area:");
		for(i=0; i<6; i++)
			printf(" %02X", extbuf[i]);
		printf("\n");
	}
	else {
		printf(" error(%d)\n", rc);
	}
	return 0;
}

U_BOOT_CMD(
    nd,  5,  0,  do_nand_display,
    "nd      - NAND data display\n",
    "[-i] block page\n"
    "    - NAND data display\n"
);

int do_nand_write ( cmd_tbl_t *cmdtp, int flag, int argc, char *argv[] )
{
	int b_no = -1;
	int p_no = -1;
	unsigned int src;
	unsigned int len;
	int count;
	int adr;
	int num, i, j;
	int rc;
	unsigned char extbuf[6] = {0xFF, 0xFF, 0xFB, 0x00, 0xFF, 0xFF};

/* 2007.04.17 fuj */
#ifdef CONFIG_ICX1111
	unsigned char idbuf[8];
	unsigned int  page_size;

	nandInitialize(NULL, idbuf);

	for (i=0; i<8; i++) idbuf[i] = 0;
	rc = nandReadId(0, idbuf);

	if((idbuf[3] & 0x03) == 0x01)
		page_size = 2048;
	else
		page_size = 4096;
#else
	nandInitialize(NULL);
#endif

	if (argc < 4) {
        /* invalid argument */
		printf ("Usage:\n%s\n", cmdtp->usage);
		return 1;
	}

	/* get parameters */
    if (my_atoi(argv[1], &num, 10) == 0)
		b_no = num;
    if (my_atoi(argv[2], &num, 10) == 0)
		p_no = num;
    if (my_atoi(argv[3], &num, 16) == 0)
		src = (unsigned int)num;
    if (my_atoi(argv[4], &num, 16) == 0)
		len = (unsigned int)num;

	/* check parameters */
	if ((b_no < 0) || (p_no < 0) ||
		(b_no > NAND_BLOCK_NUM) || (p_no > NAND_PAGE_NUM) ||
		!(((src >= 0x30000000) && (src < 0x34000000)) || ((src >= 0xA0000000) && (src < 0xA0040000))) ){
        /* invalid argument */
		printf ("Usage:\n%s\n", cmdtp->usage);
		return 1;
	}
/* 2007.04.17 fuj */
#ifdef CONFIG_ICX1111
	if (len < 0) len = page_size;
	count = len / page_size;
	if ((len % page_size) != 0) count++;
#else
	if (len < 0) len = NAND_PAGE_SIZE;
	count = len / NAND_PAGE_SIZE;
	if ((len % NAND_PAGE_SIZE) != 0) count++;
#endif

	if (((count + b_no + p_no) >= (NAND_BLOCK_NUM * NAND_PAGE_NUM)) ||
		((src < 0x34000000) && ((src + len) >= 0x34000000)) ||
		((src + len) >= 0xA0040000)) {
        /* invalid argument */
		printf ("Usage:\n%s\n", cmdtp->usage);
		return 1;
	}

	/* write */
	for (j=0; j<count; j++) {
		adr = b_no * NAND_PAGE_NUM + p_no;
		rc = nandWritePage(0, adr, (void *)src, extbuf);

		if(rc == 0) {
/* 2007.04.17 fuj */
#ifdef CONFIG_ICX1111
			src += page_size;
#else
			src += NAND_PAGE_SIZE;
#endif
		}
		else {
			printf(" error(%d)\n", rc);
/* 2007.04.17 fuj */
#ifdef CONFIG_ICX1111
			printf(" NAND Write Block[%d] Page[%d]->address[%08X] :",
					b_no, p_no, adr*page_size);
#else
			printf(" NAND Write Block[%d] Page[%d]->address[%08X] :",
					b_no, p_no, adr*NAND_PAGE_SIZE);
#endif
		}
	
		p_no++;
		if (p_no >= NAND_PAGE_NUM) {
			b_no++;
			p_no = 0;
		}
	}

	return 0;
}

U_BOOT_CMD(
    nw,  5,  0,  do_nand_write,
    "nw      - NAND data write\n",
    "block page src [length]\n"
    "    - NAND data write\n"
);

int do_nand_erase ( cmd_tbl_t *cmdtp, int flag, int argc, char *argv[] )
{
	int b_no = -1;
	int count = -1;
	int adr;
	int num, i, j;
	int rc;

/* 2007.04.17 fuj */
#ifdef CONFIG_ICX1111
	unsigned char idbuf[8];
	unsigned int  page_size;

	nandInitialize(NULL, idbuf);

	for (i=0; i<8; i++) idbuf[i] = 0;
	rc = nandReadId(0, idbuf);

	if((idbuf[3] & 0x03) == 0x01)
		page_size = 2048;
	else
		page_size = 4096;
#else
	nandInitialize(NULL);
#endif

	if (argc < 2) {
        /* invalid argument */
		printf ("Usage:\n%s\n", cmdtp->usage);
		return 1;
	}

	/* get parameters */
	for (i=1; i<argc; i++) {
       	if (my_atoi(argv[i], &num, 10) == 0) {
			if (b_no < 0)
				b_no = num;
			else if (count < 0)
				count = num;
		}
	}

	/* check parameters */
	if ((b_no < 0) || (b_no > NAND_BLOCK_NUM)) {
        /* invalid argument */
		printf ("Usage:\n%s\n", cmdtp->usage);
		return 1;
	}
	if (count < 0) count = 1;
	if ((count + b_no) >= NAND_BLOCK_NUM) {
        /* invalid argument */
		printf ("Usage:\n%s\n", cmdtp->usage);
		return 1;
	}

	/* display dump */
	for (j=0; j<count; j++) {
		adr = b_no * NAND_PAGE_NUM;
		rc = nandEraseBlock(0, adr);
/* 2007.04.17 fuj */
#ifdef CONFIG_ICX1111
		printf(" NAND Erase Block[%d]->address[%08X] :",
				b_no, adr*page_size);
#else
		printf(" NAND Erase Block[%d]->address[%08X] :",
				b_no, adr*NAND_PAGE_SIZE);
#endif

		if(rc == 0)
			printf(" OK\n");
		else
			printf(" error(%d)\n", rc);
		b_no++;
	}

	return 0;
}

U_BOOT_CMD(
    ne,  3,  0,  do_nand_erase,
    "ne      - NAND data erase\n",
    "block [count]\n"
    "    - NAND data erase\n"
);

#endif /* (CONFIG_COMMANDS & CFG_CMD_NANDBOOT) */
#endif /* CONFIG_ICX1087 */
